import * as i0 from '@angular/core';
import { inject, ElementRef, Renderer2, input, numberAttribute, computed, Directive } from '@angular/core';

/**
 * Directive to maintain an aspect ratio for an element.
 * The element will have its `padding-bottom` dynamically calculated
 * based on the provided aspect ratio to maintain the desired ratio.
 * The content inside the element will be positioned absolutely.
 * @group Components
 */
class RdxAspectRatioDirective {
    constructor() {
        this.element = inject(ElementRef);
        this.renderer = inject(Renderer2);
        /**
         * The desired aspect ratio (e.g., 16/9).
         * By default, it is set to 1 (which results in a square, 1:1).
         * @group Props
         * @defaultValue 1
         */
        this.ratio = input(1, { transform: numberAttribute });
        /**
         * Dynamically computed `padding-bottom` style for the element.
         * This value is calculated based on the inverse of the aspect ratio.
         *
         * If the ratio is zero, it defaults to `0%` to avoid division by zero.
         *
         */
        this.paddingBottom = computed(() => {
            const ratioValue = this.ratio();
            return `${ratioValue !== 0 ? (1 / ratioValue) * 100 : 0}%`;
        });
    }
    ngAfterViewInit() {
        const content = this.element.nativeElement.firstElementChild;
        if (content) {
            // Set the content to cover the entire element with absolute positioning
            this.renderer.setStyle(content, 'position', 'absolute');
            this.renderer.setStyle(content, 'inset', '0');
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxAspectRatioDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxAspectRatioDirective, isStandalone: true, selector: "[rdxAspectRatio]", inputs: { ratio: { classPropertyName: "ratio", publicName: "ratio", isSignal: true, isRequired: false, transformFunction: null } }, host: { properties: { "style.position": "'relative'", "style.width": "'100%'", "style.padding-bottom": "paddingBottom()" } }, exportAs: ["rdxAspectRatio"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxAspectRatioDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxAspectRatio]',
                    exportAs: 'rdxAspectRatio',
                    standalone: true,
                    host: {
                        '[style.position]': `'relative'`,
                        '[style.width]': `'100%'`,
                        '[style.padding-bottom]': 'paddingBottom()'
                    }
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { RdxAspectRatioDirective };
//# sourceMappingURL=radix-ng-primitives-aspect-ratio.mjs.map
