import * as i0 from '@angular/core';
import { forwardRef, input, booleanAttribute, output, linkedSignal, untracked, Directive, inject, ElementRef, NgZone, Input, APP_ID, Injectable, InjectionToken, computed, effect } from '@angular/core';
import { NG_VALUE_ACCESSOR, NgControl, FormControlDirective, FormControlName, NgModel } from '@angular/forms';
import { DOCUMENT } from '@angular/common';
import { Platform } from '@angular/cdk/platform';
import { getLocalTimeZone, CalendarDateTime, ZonedDateTime, getDayOfWeek, DateFormatter, createCalendar, toCalendar, CalendarDate, Time, startOfMonth, endOfMonth, today } from '@internationalized/date';

/**
 * Include in the providers section of a component which utilizes ControlValueAccessor to redundant code.
 *
 * ```ts
 * @Directive({
 *   providers: [provideValueAccessor(ExampleDirective)]
 *}
 * export class ExampleDirective{}
 * ```
 */
function provideValueAccessor(type) {
    return {
        provide: NG_VALUE_ACCESSOR,
        useExisting: forwardRef(() => type),
        multi: true
    };
}

/**
 * A reusable ControlValueAccessor implementation for form controls
 * @template T The type of the control's value
 */
class RdxControlValueAccessor {
    constructor() {
        /**
         * Input for the control's value with alias 'value'
         * @default undefined
         */
        this.valueInput = input(undefined, { alias: 'value' });
        /**
         * Input for the disabled state with alias 'disabled'
         * Uses booleanAttribute transform to convert string attributes to booleans
         * @default false
         */
        this.disabledInput = input(false, {
            alias: 'disabled',
            transform: booleanAttribute
        });
        this.valueChange = output();
        this._value = linkedSignal(() => this.valueInput());
        /**
         * Readonly access to the current value
         */
        this.value = this._value.asReadonly();
        this._disabled = linkedSignal(() => this.disabledInput());
        /**
         * Readonly access to the disabled state
         */
        this.disabled = this._disabled.asReadonly();
    }
    /**
     * Writes a new value to the control (ControlValueAccessor interface)
     * @param value The new value for the control
     */
    writeValue(value) {
        untracked(() => this._value.set(value));
    }
    /**
     * Registers a callback for when the control value changes (ControlValueAccessor interface)
     * @param fn The callback function
     */
    registerOnChange(fn) {
        this.onChange = fn;
    }
    /**
     * Registers a callback for when the control is touched (ControlValueAccessor interface)
     * @param fn The callback function
     */
    registerOnTouched(fn) {
        this.onTouched = fn;
    }
    /**
     * Sets the disabled state of the control (ControlValueAccessor interface)
     * @param isDisabled Whether the control should be disabled
     */
    setDisabledState(isDisabled) {
        this._disabled.set(isDisabled);
    }
    /**
     * Updates the control's value and triggers change detection
     * @param value The new value
     */
    setValue(value) {
        this._value.set(value);
        this.valueChange.emit(value);
        this.onChange?.(value);
    }
    markAsTouched() {
        this.onTouched?.();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxControlValueAccessor, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxControlValueAccessor, isStandalone: true, inputs: { valueInput: { classPropertyName: "valueInput", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, disabledInput: { classPropertyName: "disabledInput", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { valueChange: "valueChange" }, providers: [provideValueAccessor(RdxControlValueAccessor)], exportAs: ["rdxControlValueAccessor"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxControlValueAccessor, decorators: [{
            type: Directive,
            args: [{
                    exportAs: 'rdxControlValueAccessor',
                    providers: [provideValueAccessor(RdxControlValueAccessor)]
                }]
        }] });
/**
 * Provides a type-safe way to inject the RdxControlValueAccessor
 * @template T The type of the control's value
 * @returns An instance of RdxControlValueAccessor<T>
 */
function injectControlValueAccessor() {
    return inject(RdxControlValueAccessor);
}

/*
 * <div [rdxAutoFocus]="true"></div>
 */
class RdxAutoFocusDirective {
    constructor() {
        this.#elementRef = inject(ElementRef);
        this.#ngZone = inject(NgZone);
        this._autoSelect = false;
    }
    #elementRef;
    #ngZone;
    /**
     * @default false
     */
    set autoFocus(value) {
        if (value) {
            // Note: Running this outside Angular's zone because `element.focus()` does not trigger change detection.
            this.#ngZone.runOutsideAngular(() => 
            // Note: `element.focus()` causes re-layout which might lead to frame drops on slower devices.
            // https://gist.github.com/paulirish/5d52fb081b3570c81e3a#setting-focus
            // `setTimeout` is a macrotask executed within the current rendering frame.
            // Animation tasks are executed in the next rendering frame.
            reqAnimationFrame(() => {
                this.#elementRef.nativeElement.focus();
                if (this._autoSelect && this.#elementRef.nativeElement.select) {
                    this.#elementRef.nativeElement.select();
                }
            }));
        }
    }
    // Setter for autoSelect attribute to enable text selection when autoFocus is true.
    set autoSelect(value) {
        this._autoSelect = value;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxAutoFocusDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "16.1.0", version: "19.2.4", type: RdxAutoFocusDirective, isStandalone: true, selector: "[rdxAutoFocus]", inputs: { autoFocus: ["rdxAutoFocus", "autoFocus", booleanAttribute], autoSelect: ["autoSelect", "autoSelect", booleanAttribute] }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxAutoFocusDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxAutoFocus]',
                    standalone: true
                }]
        }], propDecorators: { autoFocus: [{
                type: Input,
                args: [{ alias: 'rdxAutoFocus', transform: booleanAttribute }]
            }], autoSelect: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }] } });
const availablePrefixes = ['moz', 'ms', 'webkit'];
function requestAnimationFramePolyfill() {
    let lastTime = 0;
    return function (callback) {
        const currTime = new Date().getTime();
        const timeToCall = Math.max(0, 16 - (currTime - lastTime));
        const id = setTimeout(() => {
            callback(currTime + timeToCall);
        }, timeToCall);
        lastTime = currTime + timeToCall;
        return id;
    };
}
// Function to get the appropriate requestAnimationFrame method with fallback to polyfill.
function getRequestAnimationFrame() {
    if (typeof window === 'undefined') {
        return () => 0;
    }
    if (window.requestAnimationFrame) {
        // https://github.com/vuejs/vue/issues/4465
        return window.requestAnimationFrame.bind(window);
    }
    const prefix = availablePrefixes.filter((key) => `${key}RequestAnimationFrame` in window)[0];
    return prefix ? window[`${prefix}RequestAnimationFrame`] : requestAnimationFramePolyfill();
}
// Get the requestAnimationFrame function or its polyfill.
const reqAnimationFrame = getRequestAnimationFrame();

/**
 * The `clamp` function restricts a number within a specified range by returning the value itself if it
 * falls within the range, or the closest boundary value if it exceeds the range.
 * @param {number} value - The `value` parameter represents the number that you want to clamp within
 * the specified range defined by `min` and `max` values.
 * @param {number} min - If the `value` parameter is less than the `min` value, the
 * function will return the `min` value.
 * @param {number} max - If the `value` parameter is greater than the `max` value,
 * the function will return `max`.
 * @returns The `clamp` function returns the value of `value` constrained within the range defined by
 * `min` and `max`.
 */
function clamp(value, min = Number.NEGATIVE_INFINITY, max = Number.POSITIVE_INFINITY) {
    return Math.min(max, Math.max(min, value));
}
/**
 * The function `roundToStepPrecision` rounds a number to a specified precision step.
 * @param {number} value - The `value` parameter is the number that you want to round to a specific
 * precision based on the `step` parameter.
 * @param {number} step - The `step` parameter in the `roundToStepPrecision` function represents the
 * interval at which you want to round the `value`. For example, if `step` is 0.5, the `value` will be
 * rounded to the nearest half.
 * @returns the `roundedValue` after rounding it to the precision specified by the `step`.
 */
function roundToStepPrecision(value, step) {
    let roundedValue = value;
    const stepString = step.toString();
    const pointIndex = stepString.indexOf('.');
    const precision = pointIndex >= 0 ? stepString.length - pointIndex : 0;
    if (precision > 0) {
        const pow = 10 ** precision;
        roundedValue = Math.round(roundedValue * pow) / pow;
    }
    return roundedValue;
}
/**
 * The function `snapValueToStep` snaps a given value to the nearest step within a specified range.
 * @param {number} value - The `value` parameter represents the number that you want to snap to the
 * nearest step value.
 * @param {number | undefined} min - The `min` parameter represents the minimum value that the `value`
 * should be snapped to. If `value` is less than `min`, it will be snapped to `min`. If `min` is not
 * provided (undefined), then the snapping will not consider a minimum value.
 * @param {number | undefined} max - The `max` parameter represents the maximum value that the `value`
 * should be snapped to. It ensures that the snapped value does not exceed this maximum value.
 * @param {number} step - The `step` parameter in the `snapValueToStep` function represents the
 * interval at which the `value` should be snapped to. It determines the granularity of the snapping
 * operation. For example, if `step` is 5, the `value` will be snapped to the nearest multiple of
 * @returns a number that has been snapped to the nearest step value within the specified range of minimum and maximum values.
 */
function snapValueToStep(value, min, max, step) {
    min = Number(min);
    max = Number(max);
    const remainder = (value - (Number.isNaN(min) ? 0 : min)) % step;
    let snappedValue = roundToStepPrecision(Math.abs(remainder) * 2 >= step
        ? value + Math.sign(remainder) * (step - Math.abs(remainder))
        : value - remainder, step);
    if (!Number.isNaN(min)) {
        if (snappedValue < min)
            snappedValue = min;
        else if (!Number.isNaN(max) && snappedValue > max)
            snappedValue = min + Math.floor(roundToStepPrecision((max - min) / step, step)) * step;
    }
    else if (!Number.isNaN(max) && snappedValue > max) {
        snappedValue = Math.floor(roundToStepPrecision(max / step, step)) * step;
    }
    // correct floating point behavior by rounding to step precision
    snappedValue = roundToStepPrecision(snappedValue, step);
    return snappedValue;
}

function injectDocument() {
    return inject(DOCUMENT);
}

class RdxFocusInitialDirective {
    constructor() {
        /** @ignore */
        this.nativeElement = inject(ElementRef).nativeElement;
    }
    /** @ignore */
    focus() {
        this.nativeElement.focus();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxFocusInitialDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxFocusInitialDirective, isStandalone: true, selector: "[rdxFocusInitial]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxFocusInitialDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxFocusInitial]'
                }]
        }] });

function getActiveElement() {
    let activeElement = document.activeElement;
    if (activeElement == null) {
        return null;
    }
    while (activeElement != null &&
        activeElement.shadowRoot != null &&
        activeElement.shadowRoot.activeElement != null) {
        activeElement = activeElement.shadowRoot.activeElement;
    }
    return activeElement;
}

/**
 * @license
 * Copyright Google LLC All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.dev/license
 */
/**
 * Keeps track of the ID count per prefix. This helps us make the IDs a bit more deterministic
 * like they were before the service was introduced. Note that ideally we wouldn't have to do
 * this, but there are some internal tests that rely on the IDs.
 */
const counters = {};
/** Service that generates unique IDs for DOM nodes. */
class _IdGenerator {
    constructor() {
        this._appId = inject(APP_ID);
    }
    /**
     * Generates a unique ID with a specific prefix.
     * @param prefix Prefix to add to the ID.
     */
    getId(prefix) {
        // Omit the app ID if it's the default `ng`. Since the vast majority of pages have one
        // Angular app on them, we can reduce the amount of breakages by not adding it.
        if (this._appId !== 'ng') {
            prefix += this._appId;
        }
        if (!Object.prototype.hasOwnProperty.call(counters, prefix)) {
            counters[prefix] = 0;
        }
        return `${prefix}${counters[prefix]++}`;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: _IdGenerator, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: _IdGenerator, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: _IdGenerator, decorators: [{
            type: Injectable,
            args: [{ providedIn: 'root' }]
        }] });

function injectNgControl(params) {
    const ngControl = inject(NgControl, { self: true, optional: true });
    if (!params?.optional && !ngControl)
        throw new Error('NgControl not found');
    if (ngControl instanceof FormControlDirective ||
        ngControl instanceof FormControlName ||
        ngControl instanceof NgModel) {
        return ngControl;
    }
    if (params?.optional) {
        return undefined;
    }
    throw new Error('NgControl is not an instance of FormControlDirective, FormControlName or NgModel');
}

function injectIsClient() {
    return inject(Platform).isBrowser;
}

// Originally based on https://github.com/puleos/object-hash v3.0.0 (MIT)
/**
 * Serializes any input value into a string for hashing.
 *
 * This method uses best efforts to generate stable serialized values.
 * However, it is not designed for security purposes.
 * Keep in mind that there is always a chance of intentional collisions caused by user input.
 *
 * @param input any value to serialize
 * @return {string} serialized string value
 */
function serialize(input) {
    if (typeof input === 'string') {
        return `'${input}'`;
    }
    return new Serializer().serialize(input);
}
const Serializer = /*@__PURE__*/ (function () {
    class Serializer {
        #context = new Map();
        compare(a, b) {
            const typeA = typeof a;
            const typeB = typeof b;
            if (typeA === 'string' && typeB === 'string') {
                return a.localeCompare(b);
            }
            if (typeA === 'number' && typeB === 'number') {
                return a - b;
            }
            return String.prototype.localeCompare.call(this.serialize(a, true), this.serialize(b, true));
        }
        serialize(value, noQuotes) {
            if (value === null) {
                return 'null';
            }
            switch (typeof value) {
                case 'string': {
                    return noQuotes ? value : `'${value}'`;
                }
                case 'bigint': {
                    return `${value}n`;
                }
                case 'object': {
                    return this.$object(value);
                }
                case 'function': {
                    return this.$function(value);
                }
            }
            return String(value);
        }
        serializeObject(object) {
            const objString = Object.prototype.toString.call(object);
            if (objString !== '[object Object]') {
                return this.serializeBuiltInType(objString.length < 10 /* '[object a]'.length === 10, the minimum */
                    ? `unknown:${objString}`
                    : objString.slice(8, -1) /* '[object '.length === 8 */, object);
            }
            const constructor = object.constructor;
            const objName = constructor === Object || constructor === undefined ? '' : constructor.name;
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-expect-error
            if (objName !== '' && globalThis[objName] === constructor) {
                return this.serializeBuiltInType(objName, object);
            }
            if ('toJSON' in object && typeof object.toJSON === 'function') {
                const json = object.toJSON();
                return (objName +
                    (json !== null && typeof json === 'object' ? this.$object(json) : `(${this.serialize(json)})`));
            }
            const keys = Object.keys(object).sort((a, b) => a.localeCompare(b));
            let content = `${objName}{`;
            for (let i = 0; i < keys.length; i++) {
                const key = keys[i];
                content += `${key}:${this.serialize(object[key])}`;
                if (i < keys.length - 1) {
                    content += ',';
                }
            }
            return content + '}';
        }
        serializeBuiltInType(type, object) {
            // eslint-disable-next-line @typescript-eslint/ban-ts-comment
            // @ts-expect-error
            const handler = this['$' + type];
            if (handler) {
                return handler.call(this, object);
            }
            if (typeof object.entries === 'function') {
                return this.serializeObjectEntries(type, object.entries());
            }
            throw new Error(`Cannot serialize ${type}`);
        }
        serializeObjectEntries(type, entries) {
            const sortedEntries = Array.from(entries).sort((a, b) => this.compare(a[0], b[0]));
            let content = `${type}{`;
            for (let i = 0; i < sortedEntries.length; i++) {
                const [key, value] = sortedEntries[i];
                content += `${this.serialize(key, true)}:${this.serialize(value)}`;
                if (i < sortedEntries.length - 1) {
                    content += ',';
                }
            }
            return content + '}';
        }
        $object(object) {
            let content = this.#context.get(object);
            if (content === undefined) {
                this.#context.set(object, `#${this.#context.size}`);
                content = this.serializeObject(object);
                this.#context.set(object, content);
            }
            return content;
        }
        $function(fn) {
            const fnStr = Function.prototype.toString.call(fn);
            if (fnStr.slice(-15 /* "[native code] }".length */) === '[native code] }') {
                return `${fn.name || ''}()[native]`;
            }
            return `${fn.name}(${fn.length})${fnStr.replace(/\s*\n\s*/g, '')}`;
        }
        $Array(arr) {
            let content = '[';
            for (let i = 0; i < arr.length; i++) {
                content += this.serialize(arr[i]);
                if (i < arr.length - 1) {
                    content += ',';
                }
            }
            return content + ']';
        }
        $Date(date) {
            try {
                return `Date(${date.toISOString()})`;
            }
            catch {
                return `Date(null)`;
            }
        }
        $ArrayBuffer(arr) {
            return `ArrayBuffer[${new Uint8Array(arr).join(',')}]`;
        }
        $Set(set) {
            return `Set${this.$Array(Array.from(set).sort((a, b) => this.compare(a, b)))}`;
        }
        $Map(map) {
            return this.serializeObjectEntries('Map', map.entries());
        }
    }
    for (const type of ['Error', 'RegExp', 'URL']) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-expect-error
        Serializer.prototype['$' + type] = function (val) {
            return `${type}(${val})`;
        };
    }
    for (const type of [
        'Int8Array',
        'Uint8Array',
        'Uint8ClampedArray',
        'Int16Array',
        'Uint16Array',
        'Int32Array',
        'Uint32Array',
        'Float32Array',
        'Float64Array'
    ]) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-expect-error
        Serializer.prototype['$' + type] = function (arr) {
            return `${type}[${arr.join(',')}]`;
        };
    }
    for (const type of ['BigInt64Array', 'BigUint64Array']) {
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        // @ts-expect-error
        Serializer.prototype['$' + type] = function (arr) {
            return `${type}[${arr.join('n,')}${arr.length > 0 ? 'n' : ''}]`;
        };
    }
    return Serializer;
})();

/**
 * Compare two objects using reference equality and stable deep hashing.
 * @param {any} object1 First object
 * @param {any} object2 Second object
 * @return {boolean} true if equal and false if not
 */
function isEqual(object1, object2) {
    if (object1 === object2) {
        return true;
    }
    if (serialize(object1) === serialize(object2)) {
        return true;
    }
    return false;
}

function isInsideForm(el) {
    if (!el || !el.nativeElement) {
        return true;
    }
    return Boolean(el.nativeElement.closest('form'));
}

function isNullish(value) {
    return value === null || value === undefined;
}

const isNumber = (v) => typeof v === 'number';

/**
 * The function `isValueEqualOrExist` checks if a value is equal to or exists in another value or
 * array.
 * @param {T | T[] | undefined} base - It represents the base value that you want to compare with the `current` value.
 * @param {T | T[] | undefined} current - The `current` parameter represents the current value that you want to compare with the `base` value or values.
 * @returns The `isValueEqualOrExist` function returns a boolean value. It checks if the `base` value
 * is equal to the `current` value or if the `current` value exists within the `base` value. The
 * function handles cases where `base` can be a single value, an array of values, or undefined.
 */
function isValueEqualOrExist(base, current) {
    if (isNullish(base))
        return false;
    if (Array.isArray(base)) {
        return base.some((val) => isEqual(val, current));
    }
    else {
        return isEqual(base, current);
    }
}

const ALT = 'Alt';
const ARROW_DOWN = 'ArrowDown';
const ARROW_LEFT = 'ArrowLeft';
const ARROW_RIGHT = 'ArrowRight';
const ARROW_UP = 'ArrowUp';
const BACKSPACE = 'Backspace';
const CAPS_LOCK = 'CapsLock';
const CONTROL = 'Control';
const DELETE = 'Delete';
const END = 'End';
const ENTER = 'Enter';
const ESCAPE = 'Escape';
const F1 = 'F1';
const F10 = 'F10';
const F11 = 'F11';
const F12 = 'F12';
const F2 = 'F2';
const F3 = 'F3';
const F4 = 'F4';
const F5 = 'F5';
const F6 = 'F6';
const F7 = 'F7';
const F8 = 'F8';
const F9 = 'F9';
const HOME = 'Home';
const META = 'Meta';
const PAGE_DOWN = 'PageDown';
const PAGE_UP = 'PageUp';
const SHIFT = 'Shift';
const SPACE = ' ';
const TAB = 'Tab';
const CTRL = 'Control';
const ASTERISK = '*';
const a = 'a';
const P = 'P';
const A = 'A';
const p = 'p';
const n = 'n';
const j = 'j';
const k = 'k';
const SPACE_CODE = 'Space';

/**
 * Creates an Angular provider that binds the given token to the existing instance
 * of the specified class. This is especially useful when you want multiple
 * tokens (or interfaces) to resolve to the same directive/component instance.
 *
 * @template T - The type associated with the injection token.
 * @param token - The InjectionToken or abstract type you want to provide.
 * @param type  - The class type whose existing instance will be used for this token.
 * @returns A Provider configuration object for Angular's DI system.
 *
 * @example
 *
 * @Directive({
 *   providers: [
 *     provideToken(RdxToggleGroupToken, RdxToggleGroupDirective),
 *     provideValueAccessor(RdxToggleGroupDirective)
 *   ]
 * })
 * export class RdxToggleGroupDirective {}
 */
function provideToken(token, type) {
    return {
        provide: token,
        useExisting: forwardRef(() => type)
    };
}

const WINDOW = new InjectionToken('An abstraction over global window object', {
    factory: () => {
        const { defaultView } = injectDocument();
        if (!defaultView) {
            throw new Error('Window is not available');
        }
        return defaultView;
    }
});
function injectWindow() {
    return inject(WINDOW);
}

// Thanks for idea.
// https://github.com/unovue/reka-ui/blob/v2/packages/core/src/shared/createContext.ts
/**
 * Creates a context with injector and provider functions for a given type
 * @template T The type of the context value
 * @param description Descriptive string for the context (used in token creation)
 * @returns A tuple containing:
 *   - injectContext: Function to retrieve the context value
 *   - provideContext: Function to create a provider for the context
 */
function createContext(description) {
    const CONTEXT_TOKEN = new InjectionToken(`${description}Context`);
    /**
     * Retrieves the context value from Angular's dependency injection
     * @param optional If true, returns null when context is not provided instead of throwing
     * @returns The context value or null if optional and not provided
     * @throws Error when context is not provided and not optional
     */
    const injectContext = (optional = false) => {
        const value = optional ? inject(CONTEXT_TOKEN, { optional: true }) : inject(CONTEXT_TOKEN);
        if (value == null && !optional) {
            throw new Error(`No context provided for token ${CONTEXT_TOKEN.toString()}`);
        }
        return value;
    };
    /**
     * Creates a provider that can be used to supply the context value
     * @param useFactory Factory function that creates the context value
     * @returns A provider that can be registered in Angular's DI system
     */
    const provideContext = (useFactory) => ({
        provide: CONTEXT_TOKEN,
        useFactory: useFactory
    });
    return [injectContext, provideContext];
}

/**
 * Splits an array into chunks of a given size.
 * @param arr The array to split.
 * @param size The size of each chunk.
 * @returns An array of arrays, where each sub-array has `size` elements from the original array.
 * @example ```ts
 * const arr = [1, 2, 3, 4, 5, 6, 7, 8];
 * const chunks = chunk(arr, 3);
 * // chunks = [[1, 2, 3], [4, 5, 6], [7, 8]]
 * ```
 */
function chunk(arr, size) {
    const result = [];
    for (let i = 0; i < arr.length; i += size)
        result.push(arr.slice(i, i + size));
    return result;
}

/**
 * Given a `DateValue` object, convert it to a native `Date` object.
 * If a timezone is provided, the date will be converted to that timezone.
 * If no timezone is provided, the date will be converted to the local timezone.
 */
function toDate(dateValue, tz = getLocalTimeZone()) {
    if (isZonedDateTime(dateValue))
        return dateValue.toDate();
    else
        return dateValue.toDate(tz);
}
function isCalendarDateTime(dateValue) {
    return dateValue instanceof CalendarDateTime;
}
function isZonedDateTime(dateValue) {
    return dateValue instanceof ZonedDateTime;
}
function hasTime(dateValue) {
    return isCalendarDateTime(dateValue) || isZonedDateTime(dateValue);
}
/**
 * Given a date, return the number of days in the month.
 */
function getDaysInMonth(date) {
    if (date instanceof Date) {
        const year = date.getFullYear();
        const month = date.getMonth() + 1;
        /**
         * By using zero as the day, we get the
         * last day of the previous month, which
         * is the month we originally passed in.
         */
        return new Date(year, month, 0).getDate();
    }
    else {
        return date.set({ day: 100 }).day;
    }
}
/**
 * Determine if a date is before the reference date.
 * @param dateToCompare - is this date before the `referenceDate`
 * @param referenceDate - is the `dateToCompare` before this date
 *
 * @see {@link isBeforeOrSame} for inclusive
 */
function isBefore(dateToCompare, referenceDate) {
    return dateToCompare.compare(referenceDate) < 0;
}
/**
 * Determine if a date is after the reference date.
 * @param dateToCompare - is this date after the `referenceDate`
 * @param referenceDate - is the `dateToCompare` after this date
 *
 * @see {@link isAfterOrSame} for inclusive
 */
function isAfter(dateToCompare, referenceDate) {
    return dateToCompare.compare(referenceDate) > 0;
}
/**
 * Determine if a date is before or the same as the reference date.
 *
 * @param dateToCompare - the date to compare
 * @param referenceDate - the reference date to make the comparison against
 *
 * @see {@link isBefore} for non-inclusive
 */
function isBeforeOrSame(dateToCompare, referenceDate) {
    return dateToCompare.compare(referenceDate) <= 0;
}
/**
 * Determine if a date is after or the same as the reference date.
 *
 * @param dateToCompare - is this date after or the same as the `referenceDate`
 * @param referenceDate - is the `dateToCompare` after or the same as this date
 *
 * @see {@link isAfter} for non-inclusive
 */
function isAfterOrSame(dateToCompare, referenceDate) {
    return dateToCompare.compare(referenceDate) >= 0;
}
/**
 * Determine if a date is inclusively between a start and end reference date.
 *
 * @param date - is this date inclusively between the `start` and `end` dates
 * @param start - the start reference date to make the comparison against
 * @param end - the end reference date to make the comparison against
 *
 * @see {@link isBetween} for non-inclusive
 */
function isBetweenInclusive(date, start, end) {
    return isAfterOrSame(date, start) && isBeforeOrSame(date, end);
}
/**
 * Determine if a date is between a start and end reference date.
 *
 * @param date - is this date between the `start` and `end` dates
 * @param start - the start reference date to make the comparison against
 * @param end - the end reference date to make the comparison against
 *
 * @see {@link isBetweenInclusive} for inclusive
 */
function isBetween(date, start, end) {
    return isAfter(date, start) && isBefore(date, end);
}
function getLastFirstDayOfWeek(date, firstDayOfWeek, locale) {
    const day = getDayOfWeek(date, locale);
    if (firstDayOfWeek > day)
        return date.subtract({ days: day + 7 - firstDayOfWeek });
    if (firstDayOfWeek === day)
        return date;
    return date.subtract({ days: day - firstDayOfWeek });
}
function getNextLastDayOfWeek(date, firstDayOfWeek, locale) {
    const day = getDayOfWeek(date, locale);
    const lastDayOfWeek = firstDayOfWeek === 0 ? 6 : firstDayOfWeek - 1;
    if (day === lastDayOfWeek)
        return date;
    if (day > lastDayOfWeek)
        return date.add({ days: 7 - day + lastDayOfWeek });
    return date.add({ days: lastDayOfWeek - day });
}
function areAllDaysBetweenValid(start, end, isUnavailable, isDisabled) {
    if (isUnavailable === undefined && isDisabled === undefined)
        return true;
    let dCurrent = start.add({ days: 1 });
    if (isDisabled?.(dCurrent) || isUnavailable?.(dCurrent))
        return false;
    const dEnd = end;
    while (dCurrent.compare(dEnd) < 0) {
        dCurrent = dCurrent.add({ days: 1 });
        if (isDisabled?.(dCurrent) || isUnavailable?.(dCurrent))
            return false;
    }
    return true;
}
/**
 * A helper function used throughout the various date builders
 * to generate a default `DateValue` using the `defaultValue`,
 * `defaultPlaceholder`, and `granularity` props.
 *
 * It's important to match the `DateValue` type being used
 * elsewhere in the builder, so they behave according to the
 * behavior the user expects based on the props they've provided.
 *
 */
function getDefaultDate(props) {
    const { defaultValue, defaultPlaceholder, granularity = 'day', locale = 'en' } = props;
    if (Array.isArray(defaultValue) && defaultValue.length)
        return defaultValue[defaultValue.length - 1].copy();
    if (defaultValue && !Array.isArray(defaultValue))
        return defaultValue.copy();
    if (defaultPlaceholder)
        return defaultPlaceholder.copy();
    const date = new Date();
    const year = date.getFullYear();
    const month = date.getMonth() + 1;
    const day = date.getDate();
    const calendarDateTimeGranularities = ['hour', 'minute', 'second'];
    const defaultFormatter = new DateFormatter(locale);
    const calendar = createCalendar(defaultFormatter.resolvedOptions().calendar);
    if (calendarDateTimeGranularities.includes(granularity ?? 'day'))
        return toCalendar(new CalendarDateTime(year, month, day, 0, 0, 0), calendar);
    return toCalendar(new CalendarDate(year, month, day), calendar);
}
function getDefaultTime(props) {
    const { defaultValue, defaultPlaceholder } = props;
    if (defaultValue) {
        return defaultValue.copy();
    }
    if (defaultPlaceholder) {
        return defaultPlaceholder.copy();
    }
    return new Time(0, 0, 0);
}

/**
 * Retrieves an array of date values representing the days between
 * the provided start and end dates.
 */
function getDaysBetween(start, end) {
    const days = [];
    let dCurrent = start.add({ days: 1 });
    const dEnd = end;
    while (dCurrent.compare(dEnd) < 0) {
        days.push(dCurrent);
        dCurrent = dCurrent.add({ days: 1 });
    }
    return days;
}
function createMonth(props) {
    const { dateObj, weekStartsOn, fixedWeeks, locale } = props;
    const daysInMonth = getDaysInMonth(dateObj);
    const datesArray = Array.from({ length: daysInMonth }, (_, i) => dateObj.set({ day: i + 1 }));
    const firstDayOfMonth = startOfMonth(dateObj);
    const lastDayOfMonth = endOfMonth(dateObj);
    const lastSunday = getLastFirstDayOfWeek(firstDayOfMonth, weekStartsOn, locale);
    const nextSaturday = getNextLastDayOfWeek(lastDayOfMonth, weekStartsOn, locale);
    const lastMonthDays = getDaysBetween(lastSunday.subtract({ days: 1 }), firstDayOfMonth);
    const nextMonthDays = getDaysBetween(lastDayOfMonth, nextSaturday.add({ days: 1 }));
    const totalDays = lastMonthDays.length + datesArray.length + nextMonthDays.length;
    if (fixedWeeks && totalDays < 42) {
        const extraDays = 42 - totalDays;
        let startFrom = nextMonthDays[nextMonthDays.length - 1];
        if (!startFrom)
            startFrom = endOfMonth(dateObj);
        const extraDaysArray = Array.from({ length: extraDays }, (_, i) => {
            const incr = i + 1;
            return startFrom.add({ days: incr });
        });
        nextMonthDays.push(...extraDaysArray);
    }
    const allDays = lastMonthDays.concat(datesArray, nextMonthDays);
    const weeks = chunk(allDays, 7);
    return {
        value: dateObj,
        dates: allDays,
        weeks: weeks
    };
}
function createMonths(props) {
    const { numberOfMonths, dateObj, ...monthProps } = props;
    const months = [];
    if (!numberOfMonths || numberOfMonths === 1) {
        months.push(createMonth({
            ...monthProps,
            dateObj
        }));
        return months;
    }
    months.push(createMonth({
        ...monthProps,
        dateObj
    }));
    // Create all the months, starting with the current month
    for (let i = 1; i < numberOfMonths; i++) {
        const nextMonth = dateObj.add({ months: i });
        months.push(createMonth({
            ...monthProps,
            dateObj: nextMonth
        }));
    }
    return months;
}

const defaultPartOptions = {
    year: 'numeric',
    month: 'numeric',
    day: 'numeric',
    hour: 'numeric',
    minute: 'numeric',
    second: 'numeric'
};
/**
 * Creates a wrapper around the `DateFormatter`, which is
 * an improved version of the {@link Intl.DateTimeFormat} API,
 * that is used internally by the various date builders to
 * easily format dates in a consistent way.
 *
 * @see [DateFormatter](https://react-spectrum.adobe.com/internationalized/date/DateFormatter.html)
 */
function createFormatter(initialLocale, opts = {}) {
    let locale = initialLocale;
    function setLocale(newLocale) {
        locale = newLocale;
    }
    function getLocale() {
        return locale;
    }
    function custom(date, options) {
        return new DateFormatter(locale, { ...opts, ...options }).format(date);
    }
    function selectedDate(date, includeTime = true) {
        if (hasTime(date) && includeTime) {
            return custom(toDate(date), {
                dateStyle: 'long',
                timeStyle: 'long'
            });
        }
        else {
            return custom(toDate(date), {
                dateStyle: 'long'
            });
        }
    }
    function fullMonthAndYear(date, options = {}) {
        return new DateFormatter(locale, { ...opts, month: 'long', year: 'numeric', ...options }).format(date);
    }
    function fullMonth(date, options = {}) {
        return new DateFormatter(locale, { ...opts, month: 'long', ...options }).format(date);
    }
    function fullYear(date, options = {}) {
        return new DateFormatter(locale, { ...opts, year: 'numeric', ...options }).format(date);
    }
    function toParts(date, options) {
        if (isZonedDateTime(date)) {
            return new DateFormatter(locale, {
                ...opts,
                ...options,
                timeZone: date.timeZone
            }).formatToParts(toDate(date));
        }
        else {
            return new DateFormatter(locale, { ...opts, ...options }).formatToParts(toDate(date));
        }
    }
    function dayOfWeek(date, length = 'narrow') {
        return new DateFormatter(locale, { ...opts, weekday: length }).format(date);
    }
    function dayPeriod(date, hourCycle = undefined) {
        const parts = new DateFormatter(locale, {
            ...opts,
            hour: 'numeric',
            minute: 'numeric',
            hourCycle: hourCycle === 24 ? 'h23' : undefined
        }).formatToParts(date);
        const value = parts.find((p) => p.type === 'dayPeriod')?.value;
        if (value === 'PM') {
            return 'PM';
        }
        return 'AM';
    }
    function part(dateObj, type, options = {}) {
        const opts = { ...defaultPartOptions, ...options };
        const parts = toParts(dateObj, opts);
        const part = parts.find((p) => p.type === type);
        return part ? part.value : '';
    }
    function getMonths() {
        const defaultDate = today(getLocalTimeZone());
        const months = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12];
        return months.map((item) => ({ label: fullMonth(toDate(defaultDate.set({ month: item }))), value: item }));
    }
    return {
        setLocale,
        getLocale,
        fullMonth,
        fullYear,
        fullMonthAndYear,
        toParts,
        custom,
        part,
        dayPeriod,
        selectedDate,
        dayOfWeek,
        getMonths
    };
}

/*
 * Implementation ported from https://github.com/melt-ui/melt-ui/blob/develop/src/lib/builders/date-field/_internal/helpers.ts
 */
const DATE_SEGMENT_PARTS = ['day', 'month', 'year'];
const TIME_SEGMENT_PARTS = ['hour', 'minute', 'second', 'dayPeriod'];
const NON_EDITABLE_SEGMENT_PARTS = ['literal', 'timeZoneName'];
const EDITABLE_SEGMENT_PARTS = [...DATE_SEGMENT_PARTS, ...TIME_SEGMENT_PARTS];
const EDITABLE_TIME_SEGMENT_PARTS = [...TIME_SEGMENT_PARTS];
const ALL_SEGMENT_PARTS = [...EDITABLE_SEGMENT_PARTS, ...NON_EDITABLE_SEGMENT_PARTS];
const ALL_EXCEPT_LITERAL_PARTS = ALL_SEGMENT_PARTS.filter((part) => part !== 'literal');
function isDateSegmentPart(part) {
    return DATE_SEGMENT_PARTS.includes(part);
}
function isTimeSegmentPart(part) {
    return TIME_SEGMENT_PARTS.includes(part);
}
function isSegmentPart(part) {
    return EDITABLE_SEGMENT_PARTS.includes(part);
}
function isAnySegmentPart(part) {
    return ALL_SEGMENT_PARTS.includes(part);
}

/*
 * Implementation ported from from from https://github.com/melt-ui/melt-ui/blob/develop/src/lib/internal/helpers/date/placeholders.ts
 */
// prettier-ignore
const supportedLocales = [
    'ach', 'af', 'am', 'an', 'ar', 'ast', 'az', 'be', 'bg', 'bn', 'br', 'bs',
    'ca', 'cak', 'ckb', 'cs', 'cy', 'da', 'de', 'dsb', 'el', 'en', 'eo', 'es',
    'et', 'eu', 'fa', 'ff', 'fi', 'fr', 'fy', 'ga', 'gd', 'gl', 'he', 'hr',
    'hsb', 'hu', 'ia', 'id', 'it', 'ja', 'ka', 'kk', 'kn', 'ko', 'lb', 'lo',
    'lt', 'lv', 'meh', 'ml', 'ms', 'nl', 'nn', 'no', 'oc', 'pl', 'pt', 'rm',
    'ro', 'ru', 'sc', 'scn', 'sk', 'sl', 'sr', 'sv', 'szl', 'tg', 'th', 'tr',
    'uk', 'zh-CN', 'zh-TW',
];
const placeholderFields = ['year', 'month', 'day'];
const placeholders = {
    ach: { year: 'mwaka', month: 'dwe', day: 'nino' },
    af: { year: 'jjjj', month: 'mm', day: 'dd' },
    am: { year: 'ዓዓዓዓ', month: 'ሚሜ', day: 'ቀቀ' },
    an: { year: 'aaaa', month: 'mm', day: 'dd' },
    ar: { year: 'سنة', month: 'شهر', day: 'يوم' },
    ast: { year: 'aaaa', month: 'mm', day: 'dd' },
    az: { year: 'iiii', month: 'aa', day: 'gg' },
    be: { year: 'гггг', month: 'мм', day: 'дд' },
    bg: { year: 'гггг', month: 'мм', day: 'дд' },
    bn: { year: 'yyyy', month: 'মিমি', day: 'dd' },
    br: { year: 'bbbb', month: 'mm', day: 'dd' },
    bs: { year: 'gggg', month: 'mm', day: 'dd' },
    ca: { year: 'aaaa', month: 'mm', day: 'dd' },
    cak: { year: 'jjjj', month: 'ii', day: "q'q'" },
    ckb: { year: 'ساڵ', month: 'مانگ', day: 'ڕۆژ' },
    cs: { year: 'rrrr', month: 'mm', day: 'dd' },
    cy: { year: 'bbbb', month: 'mm', day: 'dd' },
    da: { year: 'åååå', month: 'mm', day: 'dd' },
    de: { year: 'jjjj', month: 'mm', day: 'tt' },
    dsb: { year: 'llll', month: 'mm', day: 'źź' },
    el: { year: 'εεεε', month: 'μμ', day: 'ηη' },
    en: { year: 'yyyy', month: 'mm', day: 'dd' },
    eo: { year: 'jjjj', month: 'mm', day: 'tt' },
    es: { year: 'aaaa', month: 'mm', day: 'dd' },
    et: { year: 'aaaa', month: 'kk', day: 'pp' },
    eu: { year: 'uuuu', month: 'hh', day: 'ee' },
    fa: { year: 'سال', month: 'ماه', day: 'روز' },
    ff: { year: 'hhhh', month: 'll', day: 'ññ' },
    fi: { year: 'vvvv', month: 'kk', day: 'pp' },
    fr: { year: 'aaaa', month: 'mm', day: 'jj' },
    fy: { year: 'jjjj', month: 'mm', day: 'dd' },
    ga: { year: 'bbbb', month: 'mm', day: 'll' },
    gd: { year: 'bbbb', month: 'mm', day: 'll' },
    gl: { year: 'aaaa', month: 'mm', day: 'dd' },
    he: { year: 'שנה', month: 'חודש', day: 'יום' },
    hr: { year: 'gggg', month: 'mm', day: 'dd' },
    hsb: { year: 'llll', month: 'mm', day: 'dd' },
    hu: { year: 'éééé', month: 'hh', day: 'nn' },
    ia: { year: 'aaaa', month: 'mm', day: 'dd' },
    id: { year: 'tttt', month: 'bb', day: 'hh' },
    it: { year: 'aaaa', month: 'mm', day: 'gg' },
    ja: { year: ' 年 ', month: '月', day: '日' },
    ka: { year: 'წწწწ', month: 'თთ', day: 'რრ' },
    kk: { year: 'жжжж', month: 'аа', day: 'кк' },
    kn: { year: 'ವವವವ', month: 'ಮಿಮೀ', day: 'ದಿದಿ' },
    ko: { year: '연도', month: '월', day: '일' },
    lb: { year: 'jjjj', month: 'mm', day: 'dd' },
    lo: { year: 'ປປປປ', month: 'ດດ', day: 'ວວ' },
    lt: { year: 'mmmm', month: 'mm', day: 'dd' },
    lv: { year: 'gggg', month: 'mm', day: 'dd' },
    meh: { year: 'aaaa', month: 'mm', day: 'dd' },
    ml: { year: 'വർഷം', month: 'മാസം', day: 'തീയതി' },
    ms: { year: 'tttt', month: 'mm', day: 'hh' },
    nl: { year: 'jjjj', month: 'mm', day: 'dd' },
    nn: { year: 'åååå', month: 'mm', day: 'dd' },
    no: { year: 'åååå', month: 'mm', day: 'dd' },
    oc: { year: 'aaaa', month: 'mm', day: 'jj' },
    pl: { year: 'rrrr', month: 'mm', day: 'dd' },
    pt: { year: 'aaaa', month: 'mm', day: 'dd' },
    rm: { year: 'oooo', month: 'mm', day: 'dd' },
    ro: { year: 'aaaa', month: 'll', day: 'zz' },
    ru: { year: 'гггг', month: 'мм', day: 'дд' },
    sc: { year: 'aaaa', month: 'mm', day: 'dd' },
    scn: { year: 'aaaa', month: 'mm', day: 'jj' },
    sk: { year: 'rrrr', month: 'mm', day: 'dd' },
    sl: { year: 'llll', month: 'mm', day: 'dd' },
    sr: { year: 'гггг', month: 'мм', day: 'дд' },
    sv: { year: 'åååå', month: 'mm', day: 'dd' },
    szl: { year: 'rrrr', month: 'mm', day: 'dd' },
    tg: { year: 'сссс', month: 'мм', day: 'рр' },
    th: { year: 'ปปปป', month: 'ดด', day: 'วว' },
    tr: { year: 'yyyy', month: 'aa', day: 'gg' },
    uk: { year: 'рррр', month: 'мм', day: 'дд' },
    'zh-CN': { year: '年', month: '月', day: '日' },
    'zh-TW': { year: '年', month: '月', day: '日' }
};
function getPlaceholderObj(locale) {
    if (!isSupportedLocale(locale)) {
        const localeLanguage = getLocaleLanguage(locale);
        if (!isSupportedLocale(localeLanguage)) {
            return placeholders.en;
        }
        else {
            return placeholders[localeLanguage];
        }
    }
    else {
        return placeholders[locale];
    }
}
function getPlaceholder(field, value, locale) {
    if (isPlaceholderField(field))
        return getPlaceholderObj(locale)[field];
    if (isDefaultField(field))
        return value;
    if (isTimeField(field))
        return '––';
    return '';
}
function isSupportedLocale(locale) {
    return supportedLocales.includes(locale);
}
function isPlaceholderField(field) {
    return placeholderFields.includes(field);
}
function isTimeField(field) {
    return field === 'hour' || field === 'minute' || field === 'second';
}
function isDefaultField(field) {
    return field === 'era' || field === 'dayPeriod';
}
function getLocaleLanguage(locale) {
    if (Intl.Locale) {
        return new Intl.Locale(locale).language;
    }
    return locale.split('-')[0];
}

function getOptsByGranularity(granularity, hourCycle, isTimeValue = false) {
    const opts = {
        year: 'numeric',
        month: '2-digit',
        day: '2-digit',
        hour: '2-digit',
        minute: '2-digit',
        second: '2-digit',
        timeZoneName: 'short',
        hourCycle: normalizeHourCycle(hourCycle),
        hour12: normalizeHour12(hourCycle)
    };
    if (isTimeValue) {
        delete opts.year;
        delete opts.month;
        delete opts.day;
    }
    if (granularity === 'day') {
        delete opts.second;
        delete opts.hour;
        delete opts.minute;
        delete opts.timeZoneName;
    }
    if (granularity === 'hour') {
        delete opts.minute;
        delete opts.second;
    }
    if (granularity === 'minute')
        delete opts.second;
    return opts;
}
function normalizeDateStep(step) {
    return {
        year: step?.year ?? 1,
        month: step?.month ?? 1,
        day: step?.day ?? 1,
        hour: step?.hour ?? 1,
        minute: step?.minute ?? 1,
        second: step?.second ?? 1,
        millisecond: step?.millisecond ?? 1
    };
}
function handleCalendarInitialFocus(calendar) {
    const selectedDay = calendar.querySelector('[data-selected]');
    if (selectedDay)
        return selectedDay.focus();
    const today = calendar.querySelector('[data-today]');
    if (today)
        return today.focus();
    const firstDay = calendar.querySelector('[data-rdx-calendar-day]');
    if (firstDay)
        return firstDay.focus();
}
function normalizeHourCycle(hourCycle) {
    if (hourCycle === 24)
        return 'h23';
    if (hourCycle === 12)
        return 'h11';
    return undefined;
}
function normalizeHour12(hourCycle) {
    if (hourCycle === 24)
        return false;
    if (hourCycle === 12)
        return true;
    return undefined;
}

const calendarDateTimeGranularities = ['hour', 'minute', 'second'];
function syncTimeSegmentValues(props) {
    return Object.fromEntries(TIME_SEGMENT_PARTS.map((part) => {
        if (part === 'dayPeriod')
            return [part, props.formatter.dayPeriod(toDate(props.value))];
        return [part, props.value[part]];
    }));
}
function initializeSegmentValues(granularity) {
    const initialParts = EDITABLE_SEGMENT_PARTS.map((part) => {
        if (part === 'dayPeriod')
            return [part, 'AM'];
        return [part, null];
    }).filter(([key]) => {
        if (key === 'literal' || key === null)
            return false;
        if (granularity === 'minute' && key === 'second')
            return false;
        if (granularity === 'hour' && (key === 'second' || key === 'minute'))
            return false;
        if (granularity === 'day')
            return !calendarDateTimeGranularities.includes(key) && key !== 'dayPeriod';
        else
            return true;
    });
    return Object.fromEntries(initialParts);
}
function syncSegmentValues(props) {
    const { formatter } = props;
    const dateValues = DATE_SEGMENT_PARTS.map((part) => {
        return [part, props.value[part]];
    });
    if ('hour' in props.value) {
        const timeValues = syncTimeSegmentValues({ value: props.value, formatter });
        return { ...Object.fromEntries(dateValues), ...timeValues };
    }
    return Object.fromEntries(dateValues);
}
function createContentObj(props) {
    const { segmentValues, formatter, locale } = props;
    function getPartContent(part) {
        if ('hour' in segmentValues) {
            const value = segmentValues[part];
            if (value !== null) {
                if (part === 'day') {
                    return formatter.part(props.dateRef.set({
                        [part]: value,
                        /**
                         * Edge case for the day field:
                         *
                         * 1. If the month is filled,
                         *   we need to ensure that the day snaps to the maximum value of that month.
                         * 2. If the month is not filled,
                         *   we default to the month with the maximum number of days (here just using January, 31 days),
                         *   so that user can input any possible day.
                         */
                        month: segmentValues.month ?? 1
                    }), part, { hourCycle: normalizeHourCycle(props.hourCycle) });
                }
                return formatter.part(props.dateRef.set({ [part]: value }), part, {
                    hourCycle: normalizeHourCycle(props.hourCycle)
                });
            }
            else {
                return getPlaceholder(part, '', locale());
            }
        }
        else {
            if (isDateSegmentPart(part)) {
                const value = segmentValues[part];
                if (value !== null) {
                    if (part === 'day') {
                        return formatter.part(props.dateRef.set({
                            [part]: value,
                            // Same logic as above for the day field
                            month: segmentValues.month ?? 1
                        }), part);
                    }
                    return formatter.part(props.dateRef.set({ [part]: value }), part);
                }
                else {
                    return getPlaceholder(part, '', locale());
                }
            }
            return '';
        }
    }
    const content = Object.keys(segmentValues).reduce((obj, part) => {
        if (!isSegmentPart(part))
            return obj;
        if ('hour' in segmentValues && part === 'dayPeriod') {
            const value = segmentValues[part];
            if (value !== null)
                obj[part] = value;
            else
                obj[part] = getPlaceholder(part, 'AM', locale());
        }
        else {
            obj[part] = getPartContent(part);
        }
        return obj;
    }, {});
    return content;
}
function createContentArr(props) {
    const { granularity, formatter, contentObj, hideTimeZone, hourCycle, isTimeValue } = props;
    const parts = formatter.toParts(props.dateRef, getOptsByGranularity(granularity, hourCycle, isTimeValue));
    const segmentContentArr = parts
        .map((part) => {
        const defaultParts = ['literal', 'timeZoneName', null];
        if (defaultParts.includes(part.type) || !isSegmentPart(part.type)) {
            return {
                part: part.type,
                value: part.value
            };
        }
        return {
            part: part.type,
            value: contentObj[part.type]
        };
    })
        .filter((segment) => {
        if (segment.part === null || segment.value === null)
            return false;
        if (segment.part === 'timeZoneName' && (!isZonedDateTime(props.dateRef) || hideTimeZone))
            return false;
        return true;
    });
    return segmentContentArr;
}
function createContent(props) {
    const contentObj = createContentObj(props);
    const contentArr = createContentArr({
        contentObj,
        ...props
    });
    return {
        obj: contentObj,
        arr: contentArr
    };
}

function isSegmentNavigationKey(key) {
    if (key === ARROW_RIGHT || key === ARROW_LEFT)
        return true;
    return false;
}
function isNumberString(value) {
    if (Number.isNaN(Number.parseInt(value)))
        return false;
    return true;
}
function isAcceptableSegmentKey(key) {
    const acceptableSegmentKeys = [
        ENTER,
        ARROW_UP,
        ARROW_DOWN,
        ARROW_LEFT,
        ARROW_RIGHT,
        BACKSPACE,
        SPACE,
        'a',
        'A',
        'p',
        'P'
    ];
    if (acceptableSegmentKeys.includes(key))
        return true;
    if (isNumberString(key))
        return true;
    return false;
}
function getSegmentElements(parentElement) {
    return Array.from(parentElement.querySelectorAll('[data-rdx-date-field-segment]')).filter((item) => item.getAttribute('data-rdx-date-field-segment') !== 'literal');
}

/*
 * Implementation ported from https://github.com/melt-ui/melt-ui/blob/develop/src/lib/internal/helpers/date/types.ts
 */

// https://github.com/unovue/reka-ui/blob/v2/packages/core/src/shared/date/useDateField.ts
function commonSegmentAttrs(props) {
    return {
        role: 'spinbutton',
        contenteditable: true,
        tabindex: props.disabled ? undefined : 0,
        spellcheck: false,
        inputmode: 'numeric',
        autocorrect: 'off',
        enterkeyhint: 'next',
        style: 'caret-color: transparent;'
    };
}
function daySegmentAttrs(props) {
    const { segmentValues, placeholder } = props;
    const isEmpty = segmentValues.day === null;
    const date = segmentValues.day ? placeholder.set({ day: segmentValues.day }) : placeholder;
    const valueNow = date.day;
    const valueMin = 1;
    const valueMax = getDaysInMonth(date);
    const valueText = isEmpty ? 'Empty' : `${valueNow}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'day,',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function monthSegmentAttrs(props) {
    const { segmentValues, placeholder, formatter } = props;
    const isEmpty = segmentValues.month === null;
    const date = segmentValues.month ? placeholder.set({ month: segmentValues.month }) : placeholder;
    const valueNow = date.month;
    const valueMin = 1;
    const valueMax = 12;
    const valueText = isEmpty ? 'Empty' : `${valueNow} - ${formatter.fullMonth(toDate(date))}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'month, ',
        contenteditable: true,
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function yearSegmentAttrs(props) {
    const { segmentValues, placeholder } = props;
    const isEmpty = segmentValues.year === null;
    const date = segmentValues.year ? placeholder.set({ year: segmentValues.year }) : placeholder;
    const valueMin = 1;
    const valueMax = 9999;
    const valueNow = date.year;
    const valueText = isEmpty ? 'Empty' : `${valueNow}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'year, ',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function hourSegmentAttrs(props) {
    const { segmentValues, hourCycle, placeholder } = props;
    if (!('hour' in segmentValues) || !('hour' in placeholder))
        return {};
    const isEmpty = segmentValues.hour === null;
    const date = segmentValues.hour ? placeholder.set({ hour: segmentValues.hour }) : placeholder;
    const valueMin = hourCycle === 12 ? 1 : 0;
    const valueMax = hourCycle === 12 ? 12 : 23;
    const valueNow = date.hour;
    const valueText = isEmpty ? 'Empty' : `${valueNow} ${segmentValues.dayPeriod ?? ''}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'hour, ',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function minuteSegmentAttrs(props) {
    const { segmentValues, placeholder } = props;
    if (!('minute' in segmentValues) || !('minute' in placeholder))
        return {};
    const isEmpty = segmentValues.minute === null;
    const date = segmentValues.minute ? placeholder.set({ minute: segmentValues.minute }) : placeholder;
    const valueNow = date.minute;
    const valueMin = 0;
    const valueMax = 59;
    const valueText = isEmpty ? 'Empty' : `${valueNow}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'minute, ',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function secondSegmentAttrs(props) {
    const { segmentValues, placeholder } = props;
    if (!('second' in segmentValues) || !('second' in placeholder))
        return {};
    const isEmpty = segmentValues.second === null;
    const date = segmentValues.second ? placeholder.set({ second: segmentValues.second }) : placeholder;
    const valueNow = date.second;
    const valueMin = 0;
    const valueMax = 59;
    const valueText = isEmpty ? 'Empty' : `${valueNow}`;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'second, ',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText,
        'data-placeholder': isEmpty ? '' : undefined
    };
}
function dayPeriodSegmentAttrs(props) {
    const { segmentValues } = props;
    if (!('dayPeriod' in segmentValues))
        return {};
    const valueMin = 0;
    const valueMax = 12;
    const valueNow = segmentValues.hour ? (segmentValues.hour > 12 ? segmentValues.hour - 12 : segmentValues.hour) : 0;
    const valueText = segmentValues.dayPeriod ?? 'AM';
    return {
        ...commonSegmentAttrs(props),
        inputmode: 'text',
        'aria-label': 'AM/PM',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText
    };
}
function literalSegmentAttrs(_props) {
    return {
        'aria-hidden': true,
        'data-segment': 'literal'
    };
}
function timeZoneSegmentAttrs(props) {
    return {
        role: 'textbox',
        'aria-label': 'timezone, ',
        'data-readonly': true,
        'data-segment': 'timeZoneName',
        tabindex: props.disabled ? undefined : 0,
        style: 'caret-color: transparent;'
    };
}
function eraSegmentAttrs(props) {
    const { segmentValues, placeholder } = props;
    const valueMin = 0;
    const valueMax = 0;
    const valueNow = 0;
    const valueText = 'era' in segmentValues ? segmentValues.era : placeholder.era;
    return {
        ...commonSegmentAttrs(props),
        'aria-label': 'era',
        'aria-valuemin': valueMin,
        'aria-valuemax': valueMax,
        'aria-valuenow': valueNow,
        'aria-valuetext': valueText
    };
}
const segmentBuilders = {
    day: {
        attrs: daySegmentAttrs
    },
    month: {
        attrs: monthSegmentAttrs
    },
    year: {
        attrs: yearSegmentAttrs
    },
    hour: {
        attrs: hourSegmentAttrs
    },
    minute: {
        attrs: minuteSegmentAttrs
    },
    second: {
        attrs: secondSegmentAttrs
    },
    dayPeriod: {
        attrs: dayPeriodSegmentAttrs
    },
    literal: {
        attrs: literalSegmentAttrs
    },
    timeZoneName: {
        attrs: timeZoneSegmentAttrs
    },
    era: {
        attrs: eraSegmentAttrs
    }
};
function useDateField(props) {
    function handleSegmentClick(e) {
        const disabled = props.disabled();
        if (disabled)
            e.preventDefault();
    }
    function deleteValue(prevValue) {
        props.hasLeftFocus.set(false);
        if (prevValue === null)
            return prevValue;
        const str = prevValue.toString();
        if (str.length === 1) {
            props.modelValue.set(undefined);
            return null;
        }
        return Number.parseInt(str.slice(0, -1));
    }
    function dateTimeValueIncrementation({ e, part, dateRef, prevValue, hourCycle }) {
        const step = props.step()[part] ?? 1;
        const sign = e.key === ARROW_UP ? step : -step;
        if (prevValue === null)
            return dateRef[part];
        if (part === 'hour' && 'hour' in dateRef) {
            const cycleArgs = [
                part,
                sign,
                { hourCycle }
            ];
            return dateRef.set({ [part]: prevValue }).cycle(...cycleArgs)[part];
        }
        const cycleArgs = [part, sign];
        if (part === 'day' && props.segmentValues().month !== null)
            return dateRef
                .set({ [part]: prevValue, month: props.segmentValues().month })
                .cycle(...cycleArgs)[part];
        return dateRef.set({ [part]: prevValue }).cycle(...cycleArgs)[part];
    }
    function updateDayOrMonth(max, num, prev) {
        let moveToNext = false;
        const maxStart = Math.floor(max / 10);
        /**
         * If the user has left the segment, we want to reset the
         * `prev` value so that we can start the segment over again
         * when the user types a number.
         */
        if (props.hasLeftFocus()) {
            props.hasLeftFocus.set(false);
            prev = null;
        }
        if (prev === null) {
            /**
             * If the user types a 0 as the first number, we want
             * to keep track of that so that when they type the next
             * number, we can move to the next segment.
             */
            if (num === 0) {
                props.lastKeyZero.set(true);
                return { value: null, moveToNext };
            }
            /**
             * If the last key was a 0, or if the first number is
             * greater than the max start digit (0-3 in most cases), then
             * we want to move to the next segment, since it's not possible
             * to continue typing a valid number in this segment.
             */
            if (props.lastKeyZero() || num > maxStart) {
                // move to next
                moveToNext = true;
            }
            props.lastKeyZero.set(false);
            /**
             * If none of the above conditions are met, then we can just
             * return the number as the segment value and continue typing
             * in this segment.
             */
            return { value: num, moveToNext };
        }
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        const digits = prev.toString().length;
        const total = Number.parseInt(prev.toString() + num.toString());
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        if (digits === 2 || total > max) {
            /**
             * As we're doing elsewhere, we're checking if the number is greater
             * than the max start digit (0-3 in most months), and if so, we're
             * going to move to the next segment.
             */
            if (num > maxStart || total > max) {
                // move to next
                moveToNext = true;
            }
            return { value: num, moveToNext };
        }
        // move to next
        moveToNext = true;
        return { value: total, moveToNext };
    }
    function updateYear(num, prev) {
        let moveToNext = false;
        /**
         * If the user has left the segment, we want to reset the
         * `prev` value so that we can start the segment over again
         * when the user types a number.
         */
        // probably not implement, kind of weird
        if (props.hasLeftFocus()) {
            props.hasLeftFocus.set(false);
            prev = null;
        }
        if (prev === null)
            return { value: num === 0 ? 1 : num, moveToNext };
        const str = prev.toString() + num.toString();
        if (str.length > 4)
            return { value: num === 0 ? 1 : num, moveToNext };
        if (str.length === 4)
            moveToNext = true;
        const int = Number.parseInt(str);
        return { value: int, moveToNext };
    }
    function updateHour(num, prev) {
        const max = 24;
        let moveToNext = false;
        const maxStart = Math.floor(max / 10);
        /**
         * If the user has left the segment, we want to reset the
         * `prev` value so that we can start the segment over again
         * when the user types a number.
         */
        // probably not implement, kind of weird
        if (props.hasLeftFocus()) {
            props.hasLeftFocus.set(false);
            prev = null;
        }
        if (prev === null) {
            /**
             * If the user types a 0 as the first number, we want
             * to keep track of that so that when they type the next
             * number, we can move to the next segment.
             */
            if (num === 0) {
                props.lastKeyZero.set(true);
                return { value: 0, moveToNext };
            }
            /**
             * If the last key was a 0, or if the first number is
             * greater than the max start digit (0-3 in most cases), then
             * we want to move to the next segment, since it's not possible
             * to continue typing a valid number in this segment.
             */
            if (props.lastKeyZero() || num > maxStart) {
                // move to next
                moveToNext = true;
            }
            props.lastKeyZero.set(false);
            /**
             * If none of the above conditions are met, then we can just
             * return the number as the segment value and continue typing
             * in this segment.
             */
            return { value: num, moveToNext };
        }
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        const digits = prev.toString().length;
        const total = Number.parseInt(prev.toString() + num.toString());
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        if (digits === 2 || total > max) {
            /**
             * As we're doing elsewhere, we're checking if the number is greater
             * than the max start digit (0-3 in most months), and if so, we're
             * going to move to the next segment.
             */
            if (num > maxStart) {
                // move to next
                moveToNext = true;
            }
            return { value: num, moveToNext };
        }
        // move to next
        moveToNext = true;
        return { value: total, moveToNext };
    }
    function updateMinuteOrSecond(num, prev) {
        const max = 59;
        let moveToNext = false;
        const maxStart = Math.floor(max / 10);
        /**
         * If the user has left the segment, we want to reset the
         * `prev` value so that we can start the segment over again
         * when the user types a number.
         */
        if (props.hasLeftFocus()) {
            props.hasLeftFocus.set(false);
            prev = null;
        }
        if (prev === null) {
            /**
             * If the user types a 0 as the first number, we want
             * to keep track of that so that when they type the next
             * number, we can move to the next segment.
             */
            if (num === 0) {
                props.lastKeyZero.set(true);
                return { value: 0, moveToNext };
            }
            /**
             * If the last key was a 0, or if the first number is
             * greater than the max start digit (0-3 in most cases), then
             * we want to move to the next segment, since it's not possible
             * to continue typing a valid number in this segment.
             */
            if (props.lastKeyZero() || num > maxStart) {
                // move to next
                moveToNext = true;
            }
            props.lastKeyZero.set(false);
            /**
             * If none of the above conditions are met, then we can just
             * return the number as the segment value and continue typing
             * in this segment.
             */
            return { value: num, moveToNext };
        }
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        const digits = prev.toString().length;
        const total = Number.parseInt(prev.toString() + num.toString());
        /**
         * If the number of digits is 2, or if the total with the existing digit
         * and the pressed digit is greater than the maximum value for this
         * month, then we will reset the segment as if the user had pressed the
         * backspace key and then typed the number.
         */
        if (digits === 2 || total > max) {
            /**
             * As we're doing elsewhere, we're checking if the number is greater
             * than the max start digit (0-3 in most months), and if so, we're
             * going to move to the next segment.
             */
            if (num > maxStart) {
                // move to next
                moveToNext = true;
            }
            return { value: num, moveToNext };
        }
        // move to next
        moveToNext = true;
        return { value: total, moveToNext };
    }
    function minuteSecondIncrementation({ e, part, dateRef, prevValue }) {
        const step = props.step()[part] ?? 1;
        const sign = e.key === ARROW_UP ? step : -step;
        const min = 0;
        const max = 59;
        if (prevValue === null)
            return sign > 0 ? min : max;
        const cycleArgs = [part, sign];
        return dateRef.set({ [part]: prevValue }).cycle(...cycleArgs)[part];
    }
    const attributes = computed(() => segmentBuilders[props.part]?.attrs({
        disabled: props.disabled(),
        placeholder: props.placeholder(),
        hourCycle: props.hourCycle,
        segmentValues: props.segmentValues(),
        formatter: props.formatter
    }) ?? {});
    function handleMonthSegmentKeydown(e) {
        if (!isAcceptableSegmentKey(e.key) || isSegmentNavigationKey(e.key))
            return;
        const prevValue = props.segmentValues().month;
        if (e.key === ARROW_DOWN || e.key === ARROW_UP) {
            props.segmentValues.update((prev) => ({
                ...prev,
                month: dateTimeValueIncrementation({
                    e,
                    part: 'month',
                    dateRef: props.placeholder(),
                    prevValue
                })
            }));
            return;
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const { value, moveToNext } = updateDayOrMonth(12, num, prevValue);
            props.segmentValues.update((prev) => ({ ...prev, month: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, month: deleteValue(prevValue) }));
        }
    }
    function handleDaySegmentKeydown(e) {
        if (!isAcceptableSegmentKey(e.key) || isSegmentNavigationKey(e.key))
            return;
        const prevValue = props.segmentValues().day;
        if (e.key === ARROW_DOWN || e.key === ARROW_UP) {
            props.segmentValues.update((prev) => ({
                ...prev,
                day: dateTimeValueIncrementation({
                    e,
                    part: 'day',
                    dateRef: props.placeholder(),
                    prevValue
                })
            }));
            return;
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const segmentMonthValue = props.segmentValues().month;
            const daysInMonth = segmentMonthValue
                ? getDaysInMonth(props.placeholder().set({ month: segmentMonthValue }))
                : getDaysInMonth(props.placeholder());
            const { value, moveToNext } = updateDayOrMonth(daysInMonth, num, prevValue);
            props.segmentValues.update((prev) => ({ ...prev, day: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, day: deleteValue(prevValue) }));
        }
    }
    function handleYearSegmentKeydown(e) {
        if (!isAcceptableSegmentKey(e.key) || isSegmentNavigationKey(e.key))
            return;
        const prevValue = props.segmentValues().year;
        if (e.key === ARROW_DOWN || e.key === ARROW_UP) {
            props.segmentValues.update((prev) => ({
                ...prev,
                year: dateTimeValueIncrementation({
                    e,
                    part: 'year',
                    dateRef: props.placeholder(),
                    prevValue
                })
            }));
            return;
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const { value, moveToNext } = updateYear(num, prevValue);
            props.segmentValues.update((prev) => ({ ...prev, year: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, year: deleteValue(prevValue) }));
        }
    }
    function handleHourSegmentKeydown(e) {
        const dateRef = props.placeholder();
        const values = props.segmentValues();
        if (!isAcceptableSegmentKey(e.key) ||
            isSegmentNavigationKey(e.key) ||
            !('hour' in dateRef) ||
            !('hour' in values))
            return;
        const prevValue = values.hour;
        const hourCycle = props.hourCycle;
        if (e.key === ARROW_UP || e.key === ARROW_DOWN) {
            props.segmentValues.update((prev) => ({
                ...prev,
                hour: dateTimeValueIncrementation({
                    e,
                    part: 'hour',
                    dateRef: props.placeholder(),
                    prevValue,
                    hourCycle
                })
            }));
            if ('dayPeriod' in props.segmentValues() && values.hour != null) {
                if (values.hour < 12)
                    props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'AM' }));
                else if (values.hour)
                    props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'PM' }));
            }
            return;
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const { value, moveToNext } = updateHour(num, prevValue);
            if ('dayPeriod' in props.segmentValues() && value && value > 12)
                props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'AM' }));
            else if ('dayPeriod' in props.segmentValues() && value)
                props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'PM' }));
            props.segmentValues.update((prev) => ({ ...prev, hour: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, hour: deleteValue(prevValue) }));
        }
    }
    function handleMinuteSegmentKeydown(e) {
        const dateRef = props.placeholder();
        const values = props.segmentValues();
        if (!isAcceptableSegmentKey(e.key) ||
            isSegmentNavigationKey(e.key) ||
            !('minute' in dateRef) ||
            !('minute' in values))
            return;
        const prevValue = values.minute;
        if (e.key === ARROW_UP || e.key === ARROW_DOWN) {
            props.segmentValues.update((prev) => ({
                ...prev,
                minute: minuteSecondIncrementation({
                    e,
                    part: 'minute',
                    dateRef: props.placeholder(),
                    prevValue
                })
            }));
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const { value, moveToNext } = updateMinuteOrSecond(num, prevValue);
            props.segmentValues.update((prev) => ({ ...prev, minute: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, minute: deleteValue(prevValue) }));
        }
    }
    function handleSecondSegmentKeydown(e) {
        const dateRef = props.placeholder();
        const values = props.segmentValues();
        if (!isAcceptableSegmentKey(e.key) ||
            isSegmentNavigationKey(e.key) ||
            !('second' in dateRef) ||
            !('second' in values))
            return;
        const prevValue = values.second;
        if (e.key === ARROW_UP || e.key === ARROW_DOWN) {
            props.segmentValues.update((prev) => ({
                ...prev,
                second: minuteSecondIncrementation({
                    e,
                    part: 'second',
                    dateRef: props.placeholder(),
                    prevValue
                })
            }));
        }
        if (isNumberString(e.key)) {
            const num = Number.parseInt(e.key);
            const { value, moveToNext } = updateMinuteOrSecond(num, prevValue);
            props.segmentValues.update((prev) => ({ ...prev, second: value }));
            if (moveToNext)
                props.focusNext();
        }
        if (e.key === BACKSPACE) {
            props.hasLeftFocus.set(false);
            props.segmentValues.update((prev) => ({ ...prev, second: deleteValue(prevValue) }));
        }
    }
    function handleDayPeriodSegmentKeydown(e) {
        if (((!isAcceptableSegmentKey(e.key) || isSegmentNavigationKey(e.key)) && e.key !== 'a' && e.key !== 'p') ||
            !('hour' in props.placeholder()) ||
            !('dayPeriod' in props.segmentValues()))
            return;
        const values = props.segmentValues();
        if (e.key === ARROW_UP || e.key === ARROW_DOWN) {
            if (values.dayPeriod === 'AM') {
                props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'PM' }));
                props.segmentValues.update((prev) => ({ ...prev, hour: values.hour + 12 }));
                return;
            }
            props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'AM' }));
            props.segmentValues.update((prev) => ({ ...prev, hour: values.hour - 12 }));
            return;
        }
        if (['a', 'A'].includes(e.key) && values.dayPeriod !== 'AM') {
            props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'AM' }));
            props.segmentValues.update((prev) => ({ ...prev, hour: values.hour - 12 }));
            return;
        }
        if (['p', 'P'].includes(e.key) && values.dayPeriod !== 'PM') {
            props.segmentValues.update((prev) => ({ ...prev, dayPeriod: 'PM' }));
            props.segmentValues.update((prev) => ({ ...prev, hour: values.hour + 12 }));
        }
    }
    function handleSegmentKeydown(e) {
        const disabled = props.disabled();
        const readonly = props.readonly();
        if (e.key !== TAB)
            e.preventDefault();
        if (disabled || readonly)
            return;
        const segmentKeydownHandlers = {
            month: handleMonthSegmentKeydown,
            day: handleDaySegmentKeydown,
            year: handleYearSegmentKeydown,
            hour: handleHourSegmentKeydown,
            minute: handleMinuteSegmentKeydown,
            second: handleSecondSegmentKeydown,
            dayPeriod: handleDayPeriodSegmentKeydown,
            timeZoneName: () => { }
        };
        segmentKeydownHandlers[props.part](e);
        if (![ARROW_LEFT, ARROW_RIGHT].includes(e.key) &&
            e.key !== TAB &&
            e.key !== SHIFT &&
            isAcceptableSegmentKey(e.key)) {
            if (Object.values(props.segmentValues()).every((item) => item !== null)) {
                const updateObject = { ...props.segmentValues() };
                let dateRef = props.placeholder().copy();
                Object.keys(updateObject).forEach((part) => {
                    const value = updateObject[part];
                    dateRef = dateRef.set({ [part]: value });
                });
                props.modelValue.set(dateRef.copy());
            }
        }
    }
    return {
        handleSegmentClick,
        handleSegmentKeydown,
        attributes
    };
}

var RdxPositionSide;
(function (RdxPositionSide) {
    RdxPositionSide["Top"] = "top";
    RdxPositionSide["Right"] = "right";
    RdxPositionSide["Bottom"] = "bottom";
    RdxPositionSide["Left"] = "left";
})(RdxPositionSide || (RdxPositionSide = {}));
var RdxPositionAlign;
(function (RdxPositionAlign) {
    RdxPositionAlign["Start"] = "start";
    RdxPositionAlign["Center"] = "center";
    RdxPositionAlign["End"] = "end";
})(RdxPositionAlign || (RdxPositionAlign = {}));

const RDX_POSITIONS = {
    [RdxPositionSide.Top]: {
        [RdxPositionAlign.Center]: {
            originX: 'center',
            originY: 'top',
            overlayX: 'center',
            overlayY: 'bottom'
        },
        [RdxPositionAlign.Start]: {
            originX: 'start',
            originY: 'top',
            overlayX: 'start',
            overlayY: 'bottom'
        },
        [RdxPositionAlign.End]: {
            originX: 'end',
            originY: 'top',
            overlayX: 'end',
            overlayY: 'bottom'
        }
    },
    [RdxPositionSide.Right]: {
        [RdxPositionAlign.Center]: {
            originX: 'end',
            originY: 'center',
            overlayX: 'start',
            overlayY: 'center'
        },
        [RdxPositionAlign.Start]: {
            originX: 'end',
            originY: 'top',
            overlayX: 'start',
            overlayY: 'top'
        },
        [RdxPositionAlign.End]: {
            originX: 'end',
            originY: 'bottom',
            overlayX: 'start',
            overlayY: 'bottom'
        }
    },
    [RdxPositionSide.Bottom]: {
        [RdxPositionAlign.Center]: {
            originX: 'center',
            originY: 'bottom',
            overlayX: 'center',
            overlayY: 'top'
        },
        [RdxPositionAlign.Start]: {
            originX: 'start',
            originY: 'bottom',
            overlayX: 'start',
            overlayY: 'top'
        },
        [RdxPositionAlign.End]: {
            originX: 'end',
            originY: 'bottom',
            overlayX: 'end',
            overlayY: 'top'
        }
    },
    [RdxPositionSide.Left]: {
        [RdxPositionAlign.Center]: {
            originX: 'start',
            originY: 'center',
            overlayX: 'end',
            overlayY: 'center'
        },
        [RdxPositionAlign.Start]: {
            originX: 'start',
            originY: 'top',
            overlayX: 'end',
            overlayY: 'top'
        },
        [RdxPositionAlign.End]: {
            originX: 'start',
            originY: 'bottom',
            overlayX: 'end',
            overlayY: 'bottom'
        }
    }
};
const RDX_POSITIONING_DEFAULTS = {
    offsets: {
        side: 4,
        align: 0
    },
    arrow: {
        width: 8,
        height: 6
    }
};

function getContentPosition(sideAndAlignWithOffsets) {
    const { side, align, sideOffset, alignOffset } = sideAndAlignWithOffsets;
    const position = {
        ...(RDX_POSITIONS[side]?.[align] ?? RDX_POSITIONS[RdxPositionSide.Top][RdxPositionAlign.Center])
    };
    if (sideOffset || alignOffset) {
        if ([RdxPositionSide.Top, RdxPositionSide.Bottom].includes(side)) {
            if (sideOffset) {
                position.offsetY = side === RdxPositionSide.Top ? -sideOffset : sideOffset;
            }
            if (alignOffset) {
                position.offsetX = alignOffset;
            }
        }
        else {
            if (sideOffset) {
                position.offsetX = side === RdxPositionSide.Left ? -sideOffset : sideOffset;
            }
            if (alignOffset) {
                position.offsetY = alignOffset;
            }
        }
    }
    return position;
}
let allPossibleConnectedPositions;
function getAllPossibleConnectedPositions() {
    if (!allPossibleConnectedPositions) {
        allPossibleConnectedPositions = new Map();
    }
    if (allPossibleConnectedPositions.size < 1) {
        for (const [side, aligns] of Object.entries(RDX_POSITIONS)) {
            for (const [align, position] of Object.entries(aligns)) {
                allPossibleConnectedPositions.set(`${side}|${align}`, position);
            }
        }
    }
    return allPossibleConnectedPositions;
}
function getSideAndAlignFromAllPossibleConnectedPositions(position) {
    const allPossibleConnectedPositions = getAllPossibleConnectedPositions();
    let sideAndAlign;
    allPossibleConnectedPositions.forEach((value, key) => {
        if (position.originX === value.originX &&
            position.originY === value.originY &&
            position.overlayX === value.overlayX &&
            position.overlayY === value.overlayY) {
            const sideAndAlignArray = key.split('|');
            sideAndAlign = {
                side: sideAndAlignArray[0],
                align: sideAndAlignArray[1]
            };
        }
    });
    if (!sideAndAlign) {
        throw Error(`[Rdx positioning] cannot infer both side and align from the given position (${JSON.stringify(position)})`);
    }
    return sideAndAlign;
}
function getArrowPositionParams(sideAndAlign, arrowWidthAndHeight, triggerWidthAndHeight) {
    const posParams = {
        top: '',
        left: '',
        transform: '',
        transformOrigin: 'center center 0px'
    };
    if ([RdxPositionSide.Top, RdxPositionSide.Bottom].includes(sideAndAlign.side)) {
        if (sideAndAlign.side === RdxPositionSide.Top) {
            posParams.top = '100%';
        }
        else {
            posParams.top = `-${arrowWidthAndHeight.height}px`;
            posParams.transform = `rotate(180deg)`;
        }
        if (sideAndAlign.align === RdxPositionAlign.Start) {
            posParams.left = `${(triggerWidthAndHeight.width - arrowWidthAndHeight.width) / 2}px`;
        }
        else if (sideAndAlign.align === RdxPositionAlign.Center) {
            posParams.left = `calc(50% - ${arrowWidthAndHeight.width / 2}px)`;
        }
        else if (sideAndAlign.align === RdxPositionAlign.End) {
            posParams.left = `calc(100% - ${(triggerWidthAndHeight.width + arrowWidthAndHeight.width) / 2}px)`;
        }
    }
    else if ([RdxPositionSide.Left, RdxPositionSide.Right].includes(sideAndAlign.side)) {
        if (sideAndAlign.side === RdxPositionSide.Left) {
            posParams.left = `calc(100% - ${arrowWidthAndHeight.width}px)`;
            posParams.transform = `rotate(-90deg)`;
            posParams.transformOrigin = 'top right 0px';
        }
        else {
            posParams.left = `0`;
            posParams.transform = `rotate(90deg)`;
            posParams.transformOrigin = 'top left 0px';
        }
        if (sideAndAlign.align === RdxPositionAlign.Start) {
            posParams.top = `${(triggerWidthAndHeight.height - arrowWidthAndHeight.width) / 2}px`;
        }
        else if (sideAndAlign.align === RdxPositionAlign.Center) {
            posParams.top = `calc(50% - ${arrowWidthAndHeight.width / 2}px)`;
        }
        else if (sideAndAlign.align === RdxPositionAlign.End) {
            posParams.top = `calc(100% - ${(triggerWidthAndHeight.height + arrowWidthAndHeight.width) / 2}px)`;
        }
    }
    return posParams;
}

// made by https://reka-ui.com/
const ignoredElement = ['INPUT', 'TEXTAREA'];
/**
 * Allow arrow navigation for every html element with data-rdx-collection-item tag
 *
 * @param e               Keyboard event
 * @param currentElement  Event initiator element or any element that wants to handle the navigation
 * @param parentElement   Parent element where contains all the collection items, this will collect every item to be used when nav
 * @param options         further options
 * @returns               the navigated html element or null if none
 */
function useArrowNavigation(e, currentElement, parentElement, options = {}) {
    if (!currentElement || (options.enableIgnoredElement && ignoredElement.includes(currentElement.nodeName)))
        return null;
    const { arrowKeyOptions = 'both', attributeName = '[data-rdx-collection-item]', itemsArray = [], loop = true, dir = 'ltr', preventScroll = true, focus = false } = options;
    const [right, left, up, down, home, end] = [
        e.key === 'ArrowRight',
        e.key === 'ArrowLeft',
        e.key === 'ArrowUp',
        e.key === 'ArrowDown',
        e.key === 'Home',
        e.key === 'End'
    ];
    const goingVertical = up || down;
    const goingHorizontal = right || left;
    if (!home &&
        !end &&
        ((!goingVertical && !goingHorizontal) ||
            (arrowKeyOptions === 'vertical' && goingHorizontal) ||
            (arrowKeyOptions === 'horizontal' && goingVertical))) {
        return null;
    }
    const allCollectionItems = parentElement
        ? Array.from(parentElement.querySelectorAll(attributeName))
        : itemsArray;
    if (!allCollectionItems.length)
        return null;
    if (preventScroll)
        e.preventDefault();
    let item = null;
    if (goingHorizontal || goingVertical) {
        const goForward = goingVertical ? down : dir === 'ltr' ? right : left;
        item = findNextFocusableElement(allCollectionItems, currentElement, {
            goForward,
            loop
        });
    }
    else if (home) {
        item = allCollectionItems.length ? allCollectionItems[0] : null;
    }
    else if (end) {
        item = allCollectionItems.length ? allCollectionItems[allCollectionItems.length - 1] : null;
    }
    if (focus)
        item?.focus();
    return item;
}
/**
 * Recursive function to find the next focusable element to avoid disabled elements
 *
 * @param elements Elements to navigate
 * @param currentElement Current active element
 * @param options
 * @param iterations
 * @returns next focusable element
 */
function findNextFocusableElement(elements, currentElement, options, iterations = elements.length) {
    if (--iterations === 0)
        return null;
    const index = elements.indexOf(currentElement);
    const newIndex = options.goForward ? index + 1 : index - 1;
    if (!options.loop && (newIndex < 0 || newIndex >= elements.length))
        return null;
    const adjustedNewIndex = (newIndex + elements.length) % elements.length;
    const candidate = elements[adjustedNewIndex];
    if (!candidate)
        return null;
    const isDisabled = candidate.hasAttribute('disabled') && candidate.getAttribute('disabled') !== 'false';
    if (isDisabled) {
        return findNextFocusableElement(elements, candidate, options, iterations);
    }
    return candidate;
}

/**
 * Creates a resize observer effect for element
 *
 * @param options Configuration options
 * @param options.injector Angular injector
 * @param options.element Signal returning the element to observe
 * @param options.onResize Callback when element is resized
 * @returns EffectRef that can be destroyed when needed
 */
function resizeEffect(options) {
    return effect((onCleanup) => {
        const elementOrRef = options.element();
        if (!elementOrRef)
            return;
        // Extract native element from ElementRef or use directly if it's HTMLElement
        const element = elementOrRef instanceof ElementRef ? elementOrRef.nativeElement : elementOrRef;
        const resizeObserver = new ResizeObserver(options.onResize);
        resizeObserver.observe(element);
        onCleanup(() => resizeObserver.disconnect());
    }, { injector: options.injector });
}

// https://ngxtension.netlify.app/utilities/signals/explicit-effect/
/**
 * This explicit effect function will take the dependencies and the function to run when the dependencies change.
 *
 * @example
 * ```typescript
 * import { watch } from 'radix-ng/primitives/core';
 *
 * const count = signal(0);
 * const state = signal('idle');
 *
 * watch([count, state], ([count, state], cleanup) => {
 *   console.log('count updated', count, state);
 *
 *   cleanup(() => {
 *     console.log('cleanup');
 *   });
 * });
 * ```
 *
 * @param deps - The dependencies that the effect will run on
 * @param fn - The function to run when the dependencies change
 * @param options - The options for the effect with the addition of defer (it allows the computation to run on first change, not immediately)
 */
function watch(deps, fn, options) {
    let defer = options && options.defer;
    return effect((onCleanup) => {
        const depValues = deps.map((s) => s());
        untracked(() => {
            if (!defer) {
                fn(depValues, onCleanup);
            }
            defer = false;
        });
    }, options);
}

/**
 * Generated bundle index. Do not edit.
 */

export { A, ALT, ARROW_DOWN, ARROW_LEFT, ARROW_RIGHT, ARROW_UP, ASTERISK, BACKSPACE, CAPS_LOCK, CONTROL, CTRL, DELETE, END, ENTER, ESCAPE, F1, F10, F11, F12, F2, F3, F4, F5, F6, F7, F8, F9, HOME, META, P, PAGE_DOWN, PAGE_UP, RDX_POSITIONING_DEFAULTS, RDX_POSITIONS, RdxAutoFocusDirective, RdxControlValueAccessor, RdxFocusInitialDirective, RdxPositionAlign, RdxPositionSide, SHIFT, SPACE, SPACE_CODE, TAB, WINDOW, _IdGenerator, a, areAllDaysBetweenValid, clamp, createContent, createContext, createFormatter, createMonth, createMonths, getActiveElement, getAllPossibleConnectedPositions, getArrowPositionParams, getContentPosition, getDaysBetween, getDaysInMonth, getDefaultDate, getDefaultTime, getLastFirstDayOfWeek, getNextLastDayOfWeek, getOptsByGranularity, getPlaceholder, getSegmentElements, getSideAndAlignFromAllPossibleConnectedPositions, handleCalendarInitialFocus, hasTime, initializeSegmentValues, injectControlValueAccessor, injectDocument, injectIsClient, injectNgControl, injectWindow, isAcceptableSegmentKey, isAfter, isAfterOrSame, isBefore, isBeforeOrSame, isBetween, isBetweenInclusive, isCalendarDateTime, isEqual, isInsideForm, isNullish, isNumber, isNumberString, isSegmentNavigationKey, isValueEqualOrExist, isZonedDateTime, j, k, n, normalizeDateStep, normalizeHour12, normalizeHourCycle, p, provideToken, provideValueAccessor, resizeEffect, roundToStepPrecision, segmentBuilders, snapValueToStep, syncSegmentValues, syncTimeSegmentValues, toDate, useArrowNavigation, useDateField, watch };
//# sourceMappingURL=radix-ng-primitives-core.mjs.map
