import * as i0 from '@angular/core';
import { signal, computed, inject, Directive, DestroyRef, Injector, RendererFactory2, runInInjectionContext, effect, Renderer2, Injectable, makeEnvironmentProviders, importProvidersFrom, input, Input, NgModule } from '@angular/core';
import { map, filter, isObservable, of, take, merge, switchMap, takeUntil } from 'rxjs';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import { Dialog, DialogModule, DIALOG_DATA } from '@angular/cdk/dialog';

const DISMISSED_VALUE = {};
function isDismissed(v) {
    return v === DISMISSED_VALUE;
}
/**
 * Represents a reference to an open dialog.
 * Provides methods and observables to interact with and monitor the dialog's state.
 * @template C - The type of the dialog's content component
 */
class RdxDialogRef {
    /**
     * @param cdkRef - Reference to the underlying CDK dialog
     * @param config - Configuration options for the dialog
     */
    constructor(cdkRef, config) {
        this.cdkRef = cdkRef;
        this.config = config;
        // state tracking
        this._openSignal = signal(true);
        this.state = computed(() => (this._openSignal() ? 'open' : 'closed'));
        this.closed$ = this.cdkRef.closed.pipe(map((res) => (isDismissed(res) ? undefined : res)));
        this.dismissed$ = this.cdkRef.closed.pipe(filter((res) => res === DISMISSED_VALUE), map(() => undefined));
        this.result$ = this.cdkRef.closed.pipe(filter((res) => !isDismissed(res)));
    }
    get instance() {
        return this.cdkRef.componentInstance;
    }
    /**
     * Attempts to dismiss the dialog
     * Checks the canClose condition before dismissing
     */
    dismiss() {
        if (!this.instance || this.config.isAlert) {
            return;
        }
        const canClose = this.config.canClose?.(this.instance) ?? true;
        const canClose$ = isObservable(canClose) ? canClose : of(canClose);
        canClose$.pipe(take(1)).subscribe((close) => {
            if (close) {
                this.cdkRef.close(DISMISSED_VALUE);
            }
        });
    }
    close(result) {
        // check if dialog is already in closing state to prevent double-closing
        if (this.state() === 'closed') {
            return;
        }
        this._openSignal.set(false);
        if (this._previousTimeout) {
            clearTimeout(this._previousTimeout);
        }
        const closeDelay = this.config.closeDelay ?? 100; // Default to 100ms if not specified
        // Actual closing happens after delay
        this._previousTimeout = setTimeout(() => {
            this.cdkRef.close(result ?? DISMISSED_VALUE);
        }, closeDelay);
    }
}

class RdxDialogCloseDirective {
    constructor() {
        this.ref = inject(RdxDialogRef);
    }
    onClick() {
        this.ref.close();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogCloseDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxDialogCloseDirective, isStandalone: true, selector: "[rdxDialogClose]", host: { listeners: { "click": "onClick()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogCloseDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxDialogClose]',
                    standalone: true,
                    host: {
                        '(click)': 'onClick()'
                    }
                }]
        }] });

const ɵdialogData = Symbol.for('rdxDialogData');
const ɵdialogResult = Symbol.for('rdxDialogResult');
function getState(open) {
    return open ? 'open' : 'closed';
}

class RdxDialogContentDirective {
    constructor() {
        this.dialogRef = inject(RdxDialogRef);
        this.destroyRef = inject(DestroyRef);
        this.isOpen = signal(true);
        this.state = computed(() => getState(this.isOpen()));
        this.dialogRef.closed$.pipe(takeUntilDestroyed(this.destroyRef)).subscribe(() => {
            this.isOpen.set(false);
        });
    }
    /**
     * Closes the dialog with a specified result.
     *
     * @param result The result to be passed back when closing the dialog
     */
    close(result) {
        this.dialogRef.close(result);
    }
    /**
     * Dismisses the dialog without a result.
     */
    dismiss() {
        this.dialogRef.dismiss();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogContentDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxDialogContentDirective, isStandalone: true, selector: "[rdxDialogContent]", host: { attributes: { "role": "dialog" }, properties: { "attr.aria-describedby": "\"true\"", "attr.aria-labelledby": "\"true\"", "attr.data-state": "state()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogContentDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxDialogContent]',
                    standalone: true,
                    host: {
                        role: 'dialog',
                        '[attr.aria-describedby]': '"true"',
                        '[attr.aria-labelledby]': '"true"',
                        '[attr.data-state]': 'state()'
                    }
                }]
        }], ctorParameters: () => [] });

class RdxDialogDescriptionDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogDescriptionDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxDialogDescriptionDirective, isStandalone: true, selector: "[rdxDialogDescription]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogDescriptionDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxDialogDescription]',
                    standalone: true
                }]
        }] });

class RdxDialogDismissDirective {
    constructor() {
        this.ref = inject(RdxDialogRef);
    }
    onClick() {
        this.ref.dismiss();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogDismissDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxDialogDismissDirective, isStandalone: true, selector: "button[rdxDialogDismiss]", host: { attributes: { "type": "button" }, listeners: { "click": "onClick()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogDismissDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'button[rdxDialogDismiss]',
                    standalone: true,
                    host: {
                        type: 'button',
                        '(click)': 'onClick()'
                    }
                }]
        }] });

class RdxDialogTitleDirective {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogTitleDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxDialogTitleDirective, isStandalone: true, selector: "[rdxDialogTitle]", ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogTitleDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxDialogTitle]',
                    standalone: true
                }]
        }] });

/**
 * Modality control: When `isModal` is set to `true`, the dialog will:
 *
 * - Have a backdrop that blocks interaction with the rest of the page
 * - Disable closing by clicking outside or pressing Escape
 * - Set `aria-modal="true"` for screen readers
 * - Automatically focus the first tabbable element in the dialog
 * - Restore focus to the element that opened the dialog when it's closed
 *
 *
 * When `isModal` is `false`, the dialog will:
 *
 * - Not have a backdrop, allowing interaction with the rest of the page
 * - Allow closing by clicking outside or pressing Escape
 * - Not set `aria-modal` attribute
 * - Not automatically manage focus
 */
class RdxDialogService {
    #cdkDialog = inject(Dialog);
    #injector = inject(Injector);
    #rendererFactory = inject(RendererFactory2);
    #renderer = this.#rendererFactory.createRenderer(null, null);
    open(config) {
        let dialogRef;
        let modeClasses = [];
        switch (config.mode) {
            case 'sheet':
                modeClasses = ['mod-sheet', 'mod-right'];
                break;
            case 'sheet-right':
                modeClasses = ['mod-sheet', 'mod-right'];
                break;
            case 'sheet-bottom':
                modeClasses = ['mod-sheet', 'mod-bottom'];
                break;
            case 'sheet-left':
                modeClasses = ['mod-sheet', 'mod-left'];
                break;
            case 'sheet-top':
                modeClasses = ['mod-sheet', 'mod-top'];
                break;
        }
        // Create a new configuration with default closeDelay if not provided
        const extendedConfig = {
            ...config,
            closeDelay: config.closeDelay ?? 0
        };
        const cdkRef = this.#cdkDialog.open(config.content, {
            ariaModal: config.modal ?? true,
            hasBackdrop: config.modal ?? true,
            data: 'data' in config ? config.data : null,
            restoreFocus: true,
            role: config.isAlert ? 'alertdialog' : 'dialog',
            disableClose: true,
            closeOnDestroy: true,
            injector: this.#injector,
            backdropClass: config.backdropClass ? config.backdropClass : 'cdk-overlay-dark-backdrop',
            panelClass: ['dialog', ...modeClasses, ...(config.panelClasses || [])],
            autoFocus: config.autoFocus === 'first-input' ? 'dialog' : (config.autoFocus ?? 'first-tabbable'),
            ariaLabel: config.ariaLabel,
            templateContext: () => ({ dialogRef: dialogRef }),
            providers: (ref) => {
                // Create dialog ref with state tracking
                dialogRef = new RdxDialogRef(ref, extendedConfig);
                // Get overlay and backdrop references
                const overlay = ref.overlayRef.overlayElement;
                const backdrop = ref.overlayRef.backdropElement;
                // Set up effect to track and update state attributes
                runInInjectionContext(this.#injector, () => {
                    effect(() => {
                        const currentState = dialogRef.state();
                        if (overlay) {
                            this.#renderer.setAttribute(overlay, 'data-state', currentState);
                        }
                        if (backdrop) {
                            this.#renderer.setAttribute(backdrop, 'data-state', currentState);
                        }
                        // For sheet dialogs, add data-side attribute
                        if (config.mode?.startsWith('sheet-')) {
                            const side = config.mode.substring(6);
                            if (overlay) {
                                this.#renderer.setAttribute(overlay, 'data-side', side);
                            }
                            if (backdrop) {
                                this.#renderer.setAttribute(backdrop, 'data-side', side);
                            }
                        }
                    });
                });
                return [
                    {
                        provide: RdxDialogRef,
                        useValue: dialogRef
                    }
                ];
            },
            // @FIXME
            ...(config.cdkConfigOverride || {})
        });
        if (cdkRef.componentRef) {
            cdkRef.componentRef.injector
                .get(Renderer2)
                .setStyle(cdkRef.componentRef.location.nativeElement, 'display', 'contents');
        }
        if (!config.isAlert) {
            merge(cdkRef.backdropClick, cdkRef.keydownEvents.pipe(filter((e) => e.key === 'Escape' && !e.defaultPrevented)))
                .pipe(filter(() => config.canCloseWithBackdrop ?? true), switchMap(() => {
                const canClose = (cdkRef.componentInstance && config.canClose?.(cdkRef.componentInstance)) ?? true;
                const canClose$ = isObservable(canClose) ? canClose : of(canClose);
                return canClose$.pipe(take(1));
            }), takeUntil(dialogRef.closed$))
                .subscribe((canClose) => {
                if (canClose) {
                    // rather than `cdkRef.close()`, closing the `dialogRef` directly
                    // ensures that the `state` is represented correctly
                    dialogRef.close(undefined);
                }
            });
        }
        return dialogRef;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogService, decorators: [{
            type: Injectable
        }] });

/**
 * Configures the RdxDialog module by providing necessary dependencies.
 *
 * This function sets up the environment providers required for the RdxDialog to function,
 * specifically importing the Angular CDK's DialogModule.
 *
 * @returns {EnvironmentProviders} An EnvironmentProviders instance containing the DialogModule.
 */
function provideRdxDialogConfig() {
    return makeEnvironmentProviders([importProvidersFrom(DialogModule)]);
}
/**
 * Provides the RdxDialogService for dependency injection.
 *
 * This function is used to make the RdxDialogService available for injection
 * in components, directives, or other services that require dialog functionality.
 *
 * @returns {Provider} A provider for the RdxDialogService.
 */
function provideRdxDialog() {
    return RdxDialogService;
}

let nextId = 0;
/**
 * @group Components
 */
class RdxDialogTriggerDirective {
    constructor() {
        this.dialogService = inject(RdxDialogService);
        /**
         * @group Props
         */
        this.id = input(`rdx-dialog-trigger-${nextId++}`);
        this.dialogId = computed(() => `rdx-dialog-${this.id()}`);
        this.isOpen = signal(false);
        this.state = computed(() => getState(this.isOpen()));
        this.currentDialogRef = null;
    }
    onClick() {
        this.currentDialogRef = this.dialogService.open({
            ...this.dialogConfig,
            content: this.dialog
        });
        this.isOpen.set(true);
        this.currentDialogRef.closed$.subscribe(() => {
            this.isOpen.set(false);
            this.currentDialogRef = null;
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogTriggerDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxDialogTriggerDirective, isStandalone: true, selector: "[rdxDialogTrigger]", inputs: { id: { classPropertyName: "id", publicName: "id", isSignal: true, isRequired: false, transformFunction: null }, dialog: { classPropertyName: "dialog", publicName: "rdxDialogTrigger", isSignal: false, isRequired: true, transformFunction: null }, dialogConfig: { classPropertyName: "dialogConfig", publicName: "rdxDialogConfig", isSignal: false, isRequired: false, transformFunction: null } }, host: { attributes: { "type": "button" }, listeners: { "click": "onClick()" }, properties: { "attr.id": "id()", "attr.aria-haspopup": "\"dialog\"", "attr.aria-expanded": "isOpen()", "attr.aria-controls": "dialogId()", "attr.data-state": "state()" } }, providers: [provideRdxDialog()], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogTriggerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxDialogTrigger]',
                    standalone: true,
                    providers: [provideRdxDialog()],
                    host: {
                        type: 'button',
                        '[attr.id]': 'id()',
                        '[attr.aria-haspopup]': '"dialog"',
                        '[attr.aria-expanded]': 'isOpen()',
                        '[attr.aria-controls]': 'dialogId()',
                        '[attr.data-state]': 'state()',
                        '(click)': 'onClick()'
                    }
                }]
        }], propDecorators: { dialog: [{
                type: Input,
                args: [{ required: true, alias: 'rdxDialogTrigger' }]
            }], dialogConfig: [{
                type: Input,
                args: [{ alias: 'rdxDialogConfig' }]
            }] } });

function injectDialogData() {
    return inject(DIALOG_DATA);
}
function injectDialogRef() {
    return inject(RdxDialogRef);
}

const _imports = [
    RdxDialogTriggerDirective,
    RdxDialogContentDirective,
    RdxDialogTitleDirective,
    RdxDialogCloseDirective,
    RdxDialogDescriptionDirective,
    RdxDialogDismissDirective
];
class RdxDialogModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogModule, imports: [RdxDialogTriggerDirective,
            RdxDialogContentDirective,
            RdxDialogTitleDirective,
            RdxDialogCloseDirective,
            RdxDialogDescriptionDirective,
            RdxDialogDismissDirective], exports: [RdxDialogTriggerDirective,
            RdxDialogContentDirective,
            RdxDialogTitleDirective,
            RdxDialogCloseDirective,
            RdxDialogDescriptionDirective,
            RdxDialogDismissDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxDialogModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [..._imports],
                    exports: [..._imports]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { DISMISSED_VALUE, RdxDialogCloseDirective, RdxDialogContentDirective, RdxDialogDescriptionDirective, RdxDialogDismissDirective, RdxDialogModule, RdxDialogRef, RdxDialogService, RdxDialogTitleDirective, RdxDialogTriggerDirective, getState, injectDialogData, injectDialogRef, provideRdxDialog, provideRdxDialogConfig };
//# sourceMappingURL=radix-ng-primitives-dialog.mjs.map
