import * as i0 from '@angular/core';
import { InjectionToken, inject, DestroyRef, signal, effect, Injectable, computed, ElementRef, input, booleanAttribute, Directive, model, numberAttribute, NgModule } from '@angular/core';
import { NumberFormatter, NumberParser } from '@internationalized/number';
import { Subject, fromEvent } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { getActiveElement, isNullish, clamp, snapValueToStep, provideToken } from '@radix-ng/primitives/core';

const NUMBER_FIELD_ROOT_CONTEXT = new InjectionToken('NUMBER_FIELD_ROOT_CONTEXT');
function injectNumberFieldRootContext() {
    return inject(NUMBER_FIELD_ROOT_CONTEXT);
}

class PressedHoldService {
    constructor() {
        this.destroyRef = inject(DestroyRef);
    }
    create(options) {
        const timeout = signal(undefined);
        const triggerHook = new Subject();
        const isPressed = signal(false);
        const resetTimeout = () => {
            const timer = timeout();
            if (timer !== undefined) {
                window.clearTimeout(timer);
                timeout.set(undefined);
            }
        };
        const onIncrementPressStart = (delay) => {
            resetTimeout();
            if (options.disabled())
                return;
            triggerHook.next();
            timeout.set(window.setTimeout(() => {
                onIncrementPressStart(60);
            }, delay));
        };
        const onPressStart = (event) => {
            if (event.button !== 0 || isPressed())
                return;
            event.preventDefault();
            isPressed.set(true);
            onIncrementPressStart(400);
        };
        const onPressRelease = () => {
            isPressed.set(false);
            resetTimeout();
        };
        effect(() => {
            // Skip SSR environments
            if (typeof window === 'undefined')
                return;
            const targetElement = options.target?.() || window;
            const destroy$ = new Subject();
            const pointerDownSub = fromEvent(targetElement, 'pointerdown')
                .pipe(takeUntil(destroy$))
                .subscribe((e) => onPressStart(e));
            const pointerUpSub = fromEvent(window, 'pointerup').pipe(takeUntil(destroy$)).subscribe(onPressRelease);
            const pointerCancelSub = fromEvent(window, 'pointercancel')
                .pipe(takeUntil(destroy$))
                .subscribe(onPressRelease);
            this.destroyRef.onDestroy(() => {
                destroy$.next();
                destroy$.complete();
                pointerDownSub.unsubscribe();
                pointerUpSub.unsubscribe();
                pointerCancelSub.unsubscribe();
            });
        });
        return {
            isPressed: isPressed.asReadonly(),
            onTrigger: (fn) => {
                const sub = triggerHook.subscribe(fn);
                this.destroyRef.onDestroy(() => sub.unsubscribe());
            }
        };
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: PressedHoldService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: PressedHoldService }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: PressedHoldService, decorators: [{
            type: Injectable
        }] });
function useNumberFormatter(locale, options = signal({})) {
    return computed(() => new NumberFormatter(locale(), options()));
}
function useNumberParser(locale, options = signal({})) {
    return computed(() => new NumberParser(locale(), options()));
}
function handleDecimalOperation(operator, value1, value2) {
    let result = operator === '+' ? value1 + value2 : value1 - value2;
    // Check if we have decimals
    if (value1 % 1 !== 0 || value2 % 1 !== 0) {
        const value1Decimal = value1.toString().split('.');
        const value2Decimal = value2.toString().split('.');
        const value1DecimalLength = (value1Decimal[1] && value1Decimal[1].length) || 0;
        const value2DecimalLength = (value2Decimal[1] && value2Decimal[1].length) || 0;
        const multiplier = 10 ** Math.max(value1DecimalLength, value2DecimalLength);
        // Transform the decimals to integers based on the precision
        value1 = Math.round(value1 * multiplier);
        value2 = Math.round(value2 * multiplier);
        // Perform the operation on integers values to make sure we don't get a fancy decimal value
        result = operator === '+' ? value1 + value2 : value1 - value2;
        // Transform the integer result back to decimal
        result /= multiplier;
    }
    return result;
}

class RdxNumberFieldDecrementDirective {
    constructor() {
        this.elementRef = inject((ElementRef));
        this.pressedHold = inject(PressedHoldService);
        this.rootContext = injectNumberFieldRootContext();
        this.disabled = input(false, { transform: booleanAttribute });
        /**
         * @ignore
         */
        this.isDisabled = computed(() => this.rootContext.disabled() || this.disabled() || this.rootContext.isDecreaseDisabled());
        /**
         * @ignore
         */
        this.useHold = this.pressedHold.create({
            target: signal(this.elementRef.nativeElement),
            disabled: this.disabled
        });
    }
    ngOnInit() {
        this.useHold.onTrigger(() => this.rootContext.handleDecrease());
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldDecrementDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxNumberFieldDecrementDirective, isStandalone: true, selector: "button[rdxNumberFieldDecrement]", inputs: { disabled: { classPropertyName: "disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null } }, host: { attributes: { "tabindex": "-1", "type": "\"button\"" }, listeners: { "contextmenu": "$event.preventDefault()" }, properties: { "attr.aria-label": "\"Decrease\"", "attr.disabled": "isDisabled() ? \"\" : undefined", "attr.data-disabled": "isDisabled() ? \"\" : undefined", "attr.data-pressed": "useHold.isPressed() ? \"true\" : undefined", "style.user-select": "useHold.isPressed() ? \"none\" : null" } }, providers: [PressedHoldService], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldDecrementDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'button[rdxNumberFieldDecrement]',
                    providers: [PressedHoldService],
                    host: {
                        tabindex: '-1',
                        type: '"button"',
                        '[attr.aria-label]': '"Decrease"',
                        '[attr.disabled]': 'isDisabled() ? "" : undefined',
                        '[attr.data-disabled]': 'isDisabled() ? "" : undefined',
                        '[attr.data-pressed]': 'useHold.isPressed() ? "true" : undefined',
                        '[style.user-select]': 'useHold.isPressed() ? "none" : null',
                        '(contextmenu)': '$event.preventDefault()'
                    }
                }]
        }] });

class RdxNumberFieldIncrementDirective {
    constructor() {
        this.elementRef = inject((ElementRef));
        this.pressedHold = inject(PressedHoldService);
        this.rootContext = injectNumberFieldRootContext();
        this.disabled = input(false, { transform: booleanAttribute });
        /**
         * @ignore
         */
        this.isDisabled = computed(() => this.rootContext.disabled() || this.disabled() || this.rootContext.isIncreaseDisabled());
        /**
         * @ignore
         */
        this.useHold = this.pressedHold.create({
            target: signal(this.elementRef.nativeElement),
            disabled: this.disabled
        });
    }
    ngOnInit() {
        this.useHold.onTrigger(() => this.rootContext.handleIncrease());
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldIncrementDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxNumberFieldIncrementDirective, isStandalone: true, selector: "button[rdxNumberFieldIncrement]", inputs: { disabled: { classPropertyName: "disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null } }, host: { attributes: { "tabindex": "-1", "type": "\"button\"" }, listeners: { "contextmenu": "$event.preventDefault()" }, properties: { "attr.aria-label": "\"Increase\"", "attr.disabled": "isDisabled() ? \"\" : undefined", "attr.data-disabled": "isDisabled() ? \"\" : undefined", "attr.data-pressed": "useHold.isPressed() ? \"true\" : undefined", "style.user-select": "useHold.isPressed() ? \"none\" : null" } }, providers: [PressedHoldService], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldIncrementDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'button[rdxNumberFieldIncrement]',
                    providers: [PressedHoldService],
                    host: {
                        tabindex: '-1',
                        type: '"button"',
                        '[attr.aria-label]': '"Increase"',
                        '[attr.disabled]': 'isDisabled() ? "" : undefined',
                        '[attr.data-disabled]': 'isDisabled() ? "" : undefined',
                        '[attr.data-pressed]': 'useHold.isPressed() ? "true" : undefined',
                        '[style.user-select]': 'useHold.isPressed() ? "none" : null',
                        '(contextmenu)': '$event.preventDefault()'
                    }
                }]
        }] });

class RdxNumberFieldInputDirective {
    constructor() {
        this.elementRef = inject((ElementRef));
        this.rootContext = injectNumberFieldRootContext();
        this.inputValue = signal(this.rootContext.textValue());
        effect(() => {
            this.inputValue.set(this.rootContext.textValue());
            this.rootContext.setInputValue(this.inputValue());
        });
    }
    ngOnInit() {
        this.rootContext.onInputElement(this.elementRef.nativeElement);
    }
    onBeforeInput(event) {
        const target = event.target;
        const nextValue = target.value.slice(0, target.selectionStart ?? undefined) +
            (event.data ?? '') +
            target.value.slice(target.selectionEnd ?? undefined);
        if (!this.rootContext.validate(nextValue)) {
            event.preventDefault();
        }
    }
    onWheelEvent(event) {
        if (this.rootContext.disableWheelChange()) {
            return;
        }
        // only handle when in focus
        if (event.target !== getActiveElement())
            return;
        // if on a trackpad, users can scroll in both X and Y at once, check the magnitude of the change
        // if it's mostly in the X direction, then just return, the user probably doesn't mean to inc/dec
        // this isn't perfect, events come in fast with small deltas and a part of the scroll may give a false indication
        // especially if the user is scrolling near 45deg
        if (Math.abs(event.deltaY) <= Math.abs(event.deltaX)) {
            return;
        }
        event.preventDefault();
        if (event.deltaY > 0) {
            this.rootContext.handleIncrease();
        }
        else if (event.deltaY < 0) {
            this.rootContext.handleDecrease();
        }
    }
    onKeydownPageUp(event) {
        event.preventDefault();
        this.rootContext.handleIncrease(10);
    }
    onKeydownPageDown(event) {
        event.preventDefault();
        this.rootContext.handleDecrease(10);
    }
    onKeydownHome(event) {
        event.preventDefault();
        this.rootContext.handleMinMaxValue('min');
    }
    onKeydownEnd(event) {
        event.preventDefault();
        this.rootContext.handleMinMaxValue('max');
    }
    onInput(event) {
        const target = event.target;
        this.rootContext.applyInputValue(target.value);
    }
    onChange() {
        this.inputValue.set(this.rootContext.textValue());
    }
    onKeydownEnter(event) {
        const target = event.target;
        this.rootContext.applyInputValue(target.value);
    }
    onKeydownUp(event) {
        event.preventDefault();
        this.rootContext.handleIncrease();
    }
    onKeydownDown(event) {
        event.preventDefault();
        this.rootContext.handleDecrease();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldInputDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxNumberFieldInputDirective, isStandalone: true, selector: "input[rdxNumberFieldInput]", host: { attributes: { "role": "spinbutton", "type": "text", "tabindex": "0", "autocomplete": "off", "autocorrect": "off", "spellcheck": "false", "aria-roledescription": "Number field" }, listeners: { "change": "onChange()", "input": "onInput($event)", "blur": "onKeydownEnter($event)", "beforeinput": "onBeforeInput($event)", "keydown.enter": "onKeydownEnter($event)", "keydown.arrowUp": "onKeydownUp($event)", "keydown.arrowDown": "onKeydownDown($event)", "keydown.home": "onKeydownHome($event)", "keydown.end": "onKeydownEnd($event)", "keydown.pageUp": "onKeydownPageUp($event)", "keydown.pageDown": "onKeydownPageDown($event)", "wheel": "onWheelEvent($event)" }, properties: { "attr.aria-valuenow": "rootContext.value()", "attr.aria-valuemin": "rootContext.min()", "attr.aria-valuemax": "rootContext.max()", "attr.inputmode": "rootContext.inputMode()", "attr.disabled": "rootContext.disabled() ? \"\" : undefined", "attr.data-disabled": "rootContext.disabled() ? \"\" : undefined", "attr.value": "inputValue()" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldInputDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'input[rdxNumberFieldInput]',
                    host: {
                        role: 'spinbutton',
                        type: 'text',
                        tabindex: '0',
                        autocomplete: 'off',
                        autocorrect: 'off',
                        spellcheck: 'false',
                        'aria-roledescription': 'Number field',
                        '[attr.aria-valuenow]': 'rootContext.value()',
                        '[attr.aria-valuemin]': 'rootContext.min()',
                        '[attr.aria-valuemax]': 'rootContext.max()',
                        '[attr.inputmode]': 'rootContext.inputMode()',
                        '[attr.disabled]': 'rootContext.disabled() ? "" : undefined',
                        '[attr.data-disabled]': 'rootContext.disabled() ? "" : undefined',
                        '[attr.value]': 'inputValue()',
                        '(change)': 'onChange()',
                        '(input)': 'onInput($event)',
                        '(blur)': 'onKeydownEnter($event)',
                        '(beforeinput)': 'onBeforeInput($event)',
                        '(keydown.enter)': 'onKeydownEnter($event)',
                        '(keydown.arrowUp)': 'onKeydownUp($event)',
                        '(keydown.arrowDown)': 'onKeydownDown($event)',
                        '(keydown.home)': 'onKeydownHome($event)',
                        '(keydown.end)': 'onKeydownEnd($event)',
                        '(keydown.pageUp)': 'onKeydownPageUp($event)',
                        '(keydown.pageDown)': 'onKeydownPageDown($event)',
                        '(wheel)': 'onWheelEvent($event)'
                    }
                }]
        }], ctorParameters: () => [] });

class RdxNumberFieldRootDirective {
    constructor() {
        this.value = model();
        this.min = input(undefined, { transform: numberAttribute });
        this.max = input(undefined, { transform: numberAttribute });
        this.step = input(1, { transform: numberAttribute });
        this.stepSnapping = input(false, { transform: booleanAttribute });
        this.disableWheelChange = input(false, { transform: booleanAttribute });
        this.locale = input('en');
        this.disabled = input(false, { transform: booleanAttribute });
        this.formatOptions = input();
        /**
         * @ignore
         */
        this.inputEl = signal(undefined);
        /**
         * @ignore
         */
        this.isDecreaseDisabled = computed(() => !isNullish(this.value()) &&
            (this.clampInputValue(this.value()) === this.min() || (this.min() && !isNaN(this.value()))
                ? handleDecimalOperation('-', this.value(), this.step()) < this.min()
                : false));
        /**
         * @ignore
         */
        this.isIncreaseDisabled = computed(() => !isNullish(this.value()) &&
            (this.clampInputValue(this.value()) === this.max() || (this.min() && !isNaN(this.value()))
                ? handleDecimalOperation('+', this.value(), this.step()) > this.max()
                : false));
        /**
         * @ignore
         */
        this.inputMode = computed(() => {
            // The inputMode attribute influences the software keyboard that is shown on touch devices.
            // Browsers and operating systems are quite inconsistent about what keys are available, however.
            // We choose between numeric and decimal based on whether we allow negative and fractional numbers,
            // and based on testing on various devices to determine what keys are available in each inputMode.
            const hasDecimals = this.numberFormatter().resolvedOptions().maximumFractionDigits > 0;
            return hasDecimals ? 'decimal' : 'numeric';
        });
        /**
         * Replace negative textValue formatted using currencySign: 'accounting'
         * with a textValue that can be announced using a minus sign.
         * @ignore
         */
        this.textValue = computed(() => isNullish(this.value()) || isNaN(this.value()) ? '' : this.textValueFormatter().format(this.value()));
        /**
         * @ignore
         */
        this.onInputElement = (el) => this.inputEl.set(el);
    }
    /**
     * @ignore
     */
    clampInputValue(val) {
        // Clamp to min and max, round to the nearest step, and round to the specified number of digits
        let clampedValue;
        if (this.step() === undefined || isNaN(this.step()) || !this.stepSnapping()) {
            clampedValue = clamp(val, this.min(), this.max());
        }
        else {
            clampedValue = snapValueToStep(val, this.min(), this.max(), this.step());
        }
        clampedValue = this.numberParser().parse(this.numberFormatter().format(clampedValue));
        return clampedValue;
    }
    ngOnInit() {
        this.numberParser = useNumberParser(this.locale, this.formatOptions);
        this.numberFormatter = useNumberFormatter(this.locale, this.formatOptions);
        this.textValueFormatter = useNumberFormatter(this.locale, this.formatOptions);
    }
    /**
     * @ignore
     */
    handleMinMaxValue(type) {
        if (type === 'min' && this.min() !== undefined) {
            this.value.set(this.clampInputValue(this.min()));
        }
        else if (type === 'max' && this.max() !== undefined) {
            this.value.set(this.clampInputValue(this.max()));
        }
    }
    /**
     * @ignore
     */
    handleDecrease(multiplier = 1) {
        this.handleChangingValue('decrease', multiplier);
    }
    /**
     * @ignore
     */
    handleIncrease(multiplier = 1) {
        this.handleChangingValue('increase', multiplier);
    }
    /**
     * @ignore
     */
    applyInputValue(val) {
        const parsedValue = this.numberParser().parse(val);
        this.value.set(isNaN(parsedValue) ? undefined : this.clampInputValue(parsedValue));
        // Set to empty state if input value is empty
        if (!val.length) {
            return this.setInputValue(val);
        }
        // if it failed to parse, then reset input to formatted version of current number
        if (isNaN(parsedValue)) {
            return this.setInputValue(this.textValue());
        }
        return this.setInputValue(this.textValue());
    }
    /**
     * @ignore
     */
    setInputValue(val) {
        if (this.inputEl()) {
            this.inputEl.update((el) => {
                if (el)
                    el.value = val;
                return el;
            });
        }
    }
    /**
     * @ignore
     */
    validate(val) {
        return this.numberParser().isValidPartialNumber(val, this.min(), this.max());
    }
    handleChangingValue(type, multiplier = 1) {
        this.inputEl()?.focus();
        const currentInputValue = this.numberParser().parse(this.inputEl()?.value ?? '');
        if (this.disabled()) {
            return;
        }
        if (isNaN(currentInputValue)) {
            this.value.set(this.min() ?? 0);
        }
        else {
            if (type === 'increase') {
                this.value.set(this.clampInputValue(currentInputValue + (this.step() ?? 1) * multiplier));
            }
            else {
                this.value.set(this.clampInputValue(currentInputValue - (this.step() ?? 1) * multiplier));
            }
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldRootDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxNumberFieldRootDirective, isStandalone: true, selector: "[rdxNumberFieldRoot]", inputs: { value: { classPropertyName: "value", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, min: { classPropertyName: "min", publicName: "min", isSignal: true, isRequired: false, transformFunction: null }, max: { classPropertyName: "max", publicName: "max", isSignal: true, isRequired: false, transformFunction: null }, step: { classPropertyName: "step", publicName: "step", isSignal: true, isRequired: false, transformFunction: null }, stepSnapping: { classPropertyName: "stepSnapping", publicName: "stepSnapping", isSignal: true, isRequired: false, transformFunction: null }, disableWheelChange: { classPropertyName: "disableWheelChange", publicName: "disableWheelChange", isSignal: true, isRequired: false, transformFunction: null }, locale: { classPropertyName: "locale", publicName: "locale", isSignal: true, isRequired: false, transformFunction: null }, disabled: { classPropertyName: "disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null }, formatOptions: { classPropertyName: "formatOptions", publicName: "formatOptions", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { value: "valueChange" }, host: { attributes: { "role": "group" }, properties: { "attr.data-disabled": "disabled() ? \"\" : undefined" } }, providers: [provideToken(NUMBER_FIELD_ROOT_CONTEXT, RdxNumberFieldRootDirective)], exportAs: ["rdxNumberFieldRoot"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldRootDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxNumberFieldRoot]',
                    exportAs: 'rdxNumberFieldRoot',
                    providers: [provideToken(NUMBER_FIELD_ROOT_CONTEXT, RdxNumberFieldRootDirective)],
                    host: {
                        role: 'group',
                        '[attr.data-disabled]': 'disabled() ? "" : undefined'
                    }
                }]
        }] });

const _imports = [
    RdxNumberFieldRootDirective,
    RdxNumberFieldInputDirective,
    RdxNumberFieldIncrementDirective,
    RdxNumberFieldDecrementDirective
];
class RdxNumberFieldModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldModule, imports: [RdxNumberFieldRootDirective,
            RdxNumberFieldInputDirective,
            RdxNumberFieldIncrementDirective,
            RdxNumberFieldDecrementDirective], exports: [RdxNumberFieldRootDirective,
            RdxNumberFieldInputDirective,
            RdxNumberFieldIncrementDirective,
            RdxNumberFieldDecrementDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxNumberFieldModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [..._imports],
                    exports: [..._imports]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { NUMBER_FIELD_ROOT_CONTEXT, PressedHoldService, RdxNumberFieldDecrementDirective, RdxNumberFieldIncrementDirective, RdxNumberFieldInputDirective, RdxNumberFieldModule, RdxNumberFieldRootDirective, handleDecimalOperation, injectNumberFieldRootContext, useNumberFormatter, useNumberParser };
//# sourceMappingURL=radix-ng-primitives-number-field.mjs.map
