import { Observable, EMPTY, of, Subject, endWith, fromEvent, filter, takeUntil, timer, race } from 'rxjs';
import * as i0 from '@angular/core';
import { InjectionToken, inject, ViewContainerRef, TemplateRef, effect, Directive } from '@angular/core';

/**
 * Ensures that the observable stream runs inside Angular's NgZone.
 *
 * This function is a higher-order function that takes an observable stream as input and ensures
 * that all emissions, errors, and completion notifications are run inside Angular's NgZone. This
 * is particularly useful for ensuring that change detection is triggered properly in Angular
 * applications.
 *
 * @template T - The type of the items emitted by the observable.
 * @param {NgZone} zone - The Angular zone to control the change detection context.
 * @returns {(source: Observable<T>) => Observable<T>} - A function that takes an observable as input
 * and returns an observable that runs inside Angular's NgZone.
 *
 * Example usage:
 *
 * const source$ = of('some value');
 * const zoned$ = source$.pipe(runInZone(zone));
 * zoned$.subscribe(value => {
 *   console.log('Value:', value);
 * });
 */
function runInZone(zone) {
    return (source) => new Observable((observer) => source.subscribe({
        next: (value) => zone.run(() => observer.next(value)),
        error: (err) => zone.run(() => observer.error(err)),
        complete: () => zone.run(() => observer.complete())
    }));
}
/**
 * Calculates the total transition duration in milliseconds for a given HTML element.
 *
 * This function retrieves the computed style of the specified element and extracts the
 * transition duration and delay properties. It then converts these values from seconds
 * to milliseconds and returns their sum, representing the total transition duration.
 *
 * @param {HTMLElement} element - The HTML element for which to calculate the transition duration.
 * @returns {number} - The total transition duration in milliseconds.
 *
 * Example usage:
 *
 * const durationMs = getTransitionDurationMs(element);
 * console.log(`Transition duration: ${durationMs} ms`);
 */
function getTransitionDurationMs(element) {
    const { transitionDelay, transitionDuration } = window.getComputedStyle(element);
    const transitionDelaySec = parseFloat(transitionDelay);
    const transitionDurationSec = parseFloat(transitionDuration);
    return (transitionDelaySec + transitionDurationSec) * 1000;
}
function triggerReflow(element) {
    return (element || document.body).getBoundingClientRect();
}

const noopFn = () => {
    /* Noop */
};
const TransitionsMap = new Map();
/**
 * Manages the presence of an element with optional transition animation.
 *
 * @template T - The type of the context object used in the transition.
 * @param {NgZone} zone - The Angular zone to control the change detection context.
 * @param {HTMLElement} element - The target HTML element to apply the transition.
 * @param {TransitionOptions<T>} options - Options for controlling the transition behavior.
 *   @param {T} [options.context] - An optional context object to pass through the transition.
 *   @param {boolean} options.animation - A flag indicating if the transition should be animated.
 *   @param {'start' | 'continue' | 'stop'} options.state - The desired state of the transition.
 * @param {TransitionStartFn<T>} startFn - A function to start the transition.
 * @returns {Observable<void>} - An observable that emits when the transition completes.
 *
 * The `usePresence` function is designed to manage the presence and visibility of an HTML element,
 * optionally applying a transition animation. It utilizes Angular's NgZone for efficient change
 * detection management and allows for different states of transitions ('start', 'continue', 'stop').
 * The function takes a start function to handle the beginning of the transition and returns an
 * observable that completes when the transition ends.
 *
 * Example usage:
 *
 * const options: TransitionOptions<MyContext> = {
 *   context: {}, // your context object
 *   animation: true,
 *   state: 'start'
 * };
 *
 * const startFn: TransitionStartFn<MyContext> = (el, animation, context) => {
 *   el.classList.add('active');
 *   return () => el.classList.remove('active');
 * };
 *
 * usePresence(zone, element, startFn, options).subscribe(() => {
 *   console.log('Transition completed');
 * });
 */
const usePresence = (zone, element, startFn, options) => {
    let context = options.context || {};
    const transitionTimerDelayMs = options.transitionTimerDelayMs ?? 5;
    const state = options.state ?? 'stop';
    const running = TransitionsMap.get(element);
    if (running) {
        switch (state) {
            case 'continue':
                return EMPTY;
            case 'stop':
                zone.run(() => running.transition$.complete());
                context = { ...running.context, ...context };
                TransitionsMap.delete(element);
                break;
        }
    }
    const endFn = startFn(element, options.animation, context) || noopFn;
    if (!options.animation || window.getComputedStyle(element).transitionProperty === 'none') {
        zone.run(() => endFn());
        return of(undefined).pipe(runInZone(zone));
    }
    const transition$ = new Subject();
    const finishTransition$ = new Subject();
    const stop$ = transition$.pipe(endWith(true));
    TransitionsMap.set(element, {
        transition$,
        complete: () => {
            finishTransition$.next();
            finishTransition$.complete();
        },
        context
    });
    const transitionDurationMs = getTransitionDurationMs(element);
    zone.runOutsideAngular(() => {
        const transitionEnd$ = fromEvent(element, 'transitionend').pipe(filter(({ target }) => target === element), takeUntil(stop$));
        const timer$ = timer(transitionDurationMs + transitionTimerDelayMs).pipe(takeUntil(stop$));
        race(timer$, transitionEnd$, finishTransition$)
            .pipe(takeUntil(stop$))
            .subscribe(() => {
            TransitionsMap.delete(element);
            zone.run(() => {
                endFn();
                transition$.next();
                transition$.complete();
            });
        });
    });
    return transition$.asObservable();
};
const completeTransition = (element) => {
    TransitionsMap.get(element)?.complete();
};

const RDX_PRESENCE_CONTEXT = new InjectionToken('RdxPresenceContext');
/**
 * Factory provider helper.
 * In your parent component/directive you can write:
 *
 *   providers: [
 *     provideRdxPresenceContext(() => ({ present: myBooleanSignal }))
 *   ]
 */
function provideRdxPresenceContext(contextFactory) {
    return { provide: RDX_PRESENCE_CONTEXT, useFactory: contextFactory };
}
class RdxPresenceDirective {
    constructor() {
        this.context = inject(RDX_PRESENCE_CONTEXT);
        this.viewContainerRef = inject(ViewContainerRef);
        this.templateRef = inject((TemplateRef));
        this.effectRef = effect((onCleanup) => {
            if (this.context.present()) {
                const view = this.viewContainerRef.createEmbeddedView(this.templateRef, { $implicit: this.context });
                onCleanup(() => view.destroy());
            }
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxPresenceDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxPresenceDirective, isStandalone: true, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxPresenceDirective, decorators: [{
            type: Directive
        }] });

// Define constants for class names
const COLLAPSE_CLASS = 'collapse';
const COLLAPSING_CLASS = 'collapsing';
const SHOW_CLASS = 'show';
/**
 * Function to handle the start of a collapsing transition.
 *
 * @param element - The HTML element to animate.
 * @param animation - Whether to use animation or not.
 * @param context - The context containing direction and dimension information.
 * @returns A function to clean up the animation.
 */
const transitionCollapsing = (element, animation, context) => {
    const { direction, dimension } = context;
    let { maxSize } = context;
    const { classList } = element;
    /**
     * Sets initial classes based on the direction.
     */
    function setInitialClasses() {
        classList.add(COLLAPSE_CLASS);
        if (direction === 'show') {
            classList.add(SHOW_CLASS);
        }
        else {
            classList.remove(SHOW_CLASS);
        }
    }
    if (!animation) {
        setInitialClasses();
        return;
    }
    if (!maxSize) {
        maxSize = measureCollapsingElementDimensionPx(element, dimension);
        context.maxSize = maxSize;
        // Fix the height before starting the animation
        element.style[dimension] = direction !== 'show' ? maxSize : '0px';
        classList.remove(COLLAPSE_CLASS, COLLAPSING_CLASS, 'show');
        triggerReflow(element);
        // Start the animation
        classList.add(COLLAPSING_CLASS);
    }
    element.style[dimension] = direction === 'show' ? maxSize : '0px';
    return () => {
        setInitialClasses();
        classList.remove(COLLAPSING_CLASS);
        element.style[dimension] = '';
    };
};
/**
 * Measures the dimension of the collapsing element in pixels.
 *
 * @param element - The HTML element to measure.
 * @param dimension - The dimension ('width' or 'height') to measure.
 * @returns The size of the dimension in pixels.
 */
function measureCollapsingElementDimensionPx(element, dimension) {
    // SSR fix
    if (typeof navigator === 'undefined') {
        return '0px';
    }
    const { classList } = element;
    const hasShownClass = classList.contains(SHOW_CLASS);
    if (!hasShownClass) {
        classList.add(SHOW_CLASS);
    }
    element.style[dimension] = '';
    const dimensionSize = element.getBoundingClientRect()[dimension] + 'px';
    if (!hasShownClass) {
        classList.remove(SHOW_CLASS);
    }
    return dimensionSize;
}

const toastFadeInTransition = (element, animation) => {
    const { classList } = element;
    if (animation) {
        classList.add('fade');
    }
    else {
        classList.add('show');
        return;
    }
    triggerReflow(element);
    classList.add('show', 'showing');
    return () => {
        classList.remove('showing');
    };
};
const toastFadeOutTransition = ({ classList }) => {
    classList.add('showing');
    return () => {
        classList.remove('show', 'showing');
    };
};

/**
 * Generated bundle index. Do not edit.
 */

export { RDX_PRESENCE_CONTEXT, RdxPresenceDirective, completeTransition, provideRdxPresenceContext, toastFadeInTransition, toastFadeOutTransition, transitionCollapsing, usePresence };
//# sourceMappingURL=radix-ng-primitives-presence.mjs.map
