import * as i0 from '@angular/core';
import { InjectionToken, inject, model, input, computed, effect, Directive, NgModule } from '@angular/core';
import { isNullish, isNumber, provideToken } from '@radix-ng/primitives/core';

const RdxProgressToken = new InjectionToken('RdxProgressDirective');
/**
 * Injects the current instance of RdxProgressRootDirective.
 * @returns The instance of RdxProgressRootDirective.
 */
function injectProgress() {
    return inject(RdxProgressToken);
}
const MIN_PERCENT = 0;
const DEFAULT_MAX = 100;
/**
 * Directive to manage progress bar state and attributes.
 *
 * This directive provides a way to create a progress bar with customizable value and max attributes.
 * It handles aria attributes for accessibility and provides different states like 'indeterminate', 'complete', and 'loading'.
 *
 * @group Components
 */
class RdxProgressRootDirective {
    constructor() {
        /**
         * The current value of the progress bar.
         * @group Props
         * @defaultValue 0
         */
        this.value = model(MIN_PERCENT);
        /**
         * The maximum value of the progress bar.
         * @defaultValue 100
         * @group Props
         */
        this.max = model(DEFAULT_MAX);
        /**
         * Function to generate the value label.
         * @group Props
         */
        this.valueLabel = input((value, max) => this.defaultGetValueLabel(value, max));
        this.label = computed(() => this.valueLabel()(this.value(), this.max()));
        this.progressState = computed(() => {
            if (isNullish(this.value())) {
                return 'indeterminate';
            }
            if (this.value() === this.max()) {
                return 'complete';
            }
            return 'loading';
        });
        effect(() => {
            const correctedValue = this.validateValue(this.value(), this.max());
            if (correctedValue != null && correctedValue !== this.value()) {
                this.value.set(correctedValue);
            }
        });
        effect(() => {
            const correctedMax = this.validateMax(this.max());
            if (correctedMax !== this.max()) {
                this.max.set(correctedMax);
            }
        });
    }
    validateValue(value, max) {
        const isValidValueError = isNullish(value) || (isNumber(value) && !Number.isNaN(value) && value <= max && value >= 0);
        if (isValidValueError)
            return value;
        console.error(`Invalid prop \`value\` of value \`${value}\` supplied to \`ProgressRoot\`. The \`value\` prop must be:
  - a positive number
  - less than the value passed to \`max\` (or ${DEFAULT_MAX} if no \`max\` prop is set)
  - \`null\`  or \`undefined\` if the progress is indeterminate.

Defaulting to \`null\`.`);
        return null;
    }
    validateMax(max) {
        const isValidMaxError = isNumber(max) && !Number.isNaN(max) && max > 0;
        if (isValidMaxError)
            return max;
        console.error(`Invalid prop \`max\` of value \`${max}\` supplied to \`ProgressRoot\`. Only numbers greater than 0 are valid max values. Defaulting to \`${DEFAULT_MAX}\`.`);
        return DEFAULT_MAX;
    }
    defaultGetValueLabel(value, max) {
        return `${Math.round((value / max) * 100)}%`;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressRootDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxProgressRootDirective, isStandalone: true, selector: "[rdxProgressRoot]", inputs: { value: { classPropertyName: "value", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, max: { classPropertyName: "max", publicName: "max", isSignal: true, isRequired: false, transformFunction: null }, valueLabel: { classPropertyName: "valueLabel", publicName: "valueLabel", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { value: "valueChange", max: "maxChange" }, host: { attributes: { "role": "progressbar", "tabindex": "-1" }, properties: { "attr.aria-valuemax": "max()", "attr.aria-valuemin": "0", "attr.aria-valuenow": "value()", "attr.aria-valuetext": "label()", "attr.aria-label": "label()", "attr.data-state": "progressState()", "attr.data-value": "value() ?? undefined", "attr.data-max": "max()" } }, providers: [provideToken(RdxProgressToken, RdxProgressRootDirective)], exportAs: ["rdxProgressRoot"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressRootDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxProgressRoot]',
                    exportAs: 'rdxProgressRoot',
                    providers: [provideToken(RdxProgressToken, RdxProgressRootDirective)],
                    host: {
                        role: 'progressbar',
                        '[attr.aria-valuemax]': 'max()',
                        '[attr.aria-valuemin]': '0',
                        '[attr.aria-valuenow]': 'value()',
                        '[attr.aria-valuetext]': 'label()',
                        '[attr.aria-label]': 'label()',
                        '[attr.data-state]': 'progressState()',
                        '[attr.data-value]': 'value() ?? undefined',
                        '[attr.data-max]': 'max()',
                        // set tab index to -1 so screen readers will read the aria-label
                        // Note: there is a known issue with JAWS that does not read progressbar aria labels on FireFox
                        tabindex: '-1'
                    }
                }]
        }], ctorParameters: () => [] });

/**
 * Directive to manage progress indicator state and attributes.
 *
 * This directive is used to display the progress indicator inside the progress bar.
 * It inherits the state and value from the `RdxProgressRootDirective`.
 */
class RdxProgressIndicatorDirective {
    constructor() {
        /**
         * This allows the directive to access the progress bar state and values.
         */
        this.progress = injectProgress();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressIndicatorDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "19.2.4", type: RdxProgressIndicatorDirective, isStandalone: true, selector: "[rdxProgressIndicator]", host: { properties: { "attr.data-state": "progress.progressState()", "attr.data-value": "progress.value()", "attr.data-max": "progress.max()" } }, exportAs: ["rdxProgressIndicator"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressIndicatorDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxProgressIndicator]',
                    exportAs: 'rdxProgressIndicator',
                    host: {
                        '[attr.data-state]': 'progress.progressState()',
                        '[attr.data-value]': 'progress.value()',
                        '[attr.data-max]': 'progress.max()'
                    }
                }]
        }] });

const _imports = [RdxProgressRootDirective, RdxProgressIndicatorDirective];
class RdxProgressModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressModule, imports: [RdxProgressRootDirective, RdxProgressIndicatorDirective], exports: [RdxProgressRootDirective, RdxProgressIndicatorDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxProgressModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [..._imports],
                    exports: [..._imports]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { RdxProgressIndicatorDirective, RdxProgressModule, RdxProgressRootDirective, RdxProgressToken, injectProgress };
//# sourceMappingURL=radix-ng-primitives-progress.mjs.map
