import * as i0 from '@angular/core';
import { inject, NgZone, ElementRef, EventEmitter, signal, booleanAttribute, Output, Input, Directive, computed } from '@angular/core';

const ENTRY_FOCUS = 'rovingFocusGroup.onEntryFocus';
const EVENT_OPTIONS = { bubbles: false, cancelable: true };
const MAP_KEY_TO_FOCUS_INTENT = {
    ArrowLeft: 'prev',
    ArrowUp: 'prev',
    ArrowRight: 'next',
    ArrowDown: 'next',
    PageUp: 'first',
    Home: 'first',
    PageDown: 'last',
    End: 'last'
};
function getDirectionAwareKey(key, dir) {
    if (dir !== 'rtl')
        return key;
    return key === 'ArrowLeft' ? 'ArrowRight' : key === 'ArrowRight' ? 'ArrowLeft' : key;
}
function getFocusIntent(event, orientation, dir) {
    const key = getDirectionAwareKey(event.key, dir);
    if (orientation === 'vertical' && ['ArrowLeft', 'ArrowRight'].includes(key))
        return undefined;
    if (orientation === 'horizontal' && ['ArrowUp', 'ArrowDown'].includes(key))
        return undefined;
    return MAP_KEY_TO_FOCUS_INTENT[key];
}
function focusFirst(candidates, preventScroll = false, rootNode) {
    const PREVIOUSLY_FOCUSED_ELEMENT = rootNode?.activeElement ?? document.activeElement;
    for (const candidate of candidates) {
        // if focus is already where we want to go, we don't want to keep going through the candidates
        if (candidate === PREVIOUSLY_FOCUSED_ELEMENT)
            return;
        candidate.focus({ preventScroll });
        if (document.activeElement !== PREVIOUSLY_FOCUSED_ELEMENT)
            return;
    }
}
/**
 * Wraps an array around itself at a given start index
 * Example: `wrapArray(['a', 'b', 'c', 'd'], 2) === ['c', 'd', 'a', 'b']`
 */
function wrapArray(array, startIndex) {
    return array.map((_, index) => array[(startIndex + index) % array.length]);
}
function generateId() {
    return `rf-item-${Math.random().toString(36).slice(2, 11)}`;
}

class RdxRovingFocusGroupDirective {
    constructor() {
        this.ngZone = inject(NgZone);
        this.elementRef = inject(ElementRef);
        this.dir = 'ltr';
        this.loop = true;
        this.preventScrollOnEntryFocus = false;
        this.entryFocus = new EventEmitter();
        this.currentTabStopIdChange = new EventEmitter();
        /** @ignore */
        this.currentTabStopId = signal(null);
        /** @ignore */
        this.focusableItems = signal([]);
        this.isClickFocus = signal(false);
        this.isTabbingBackOut = signal(false);
        this.focusableItemsCount = signal(0);
    }
    /** @ignore */
    get dataOrientation() {
        return this.orientation || 'horizontal';
    }
    /** @ignore */
    get tabIndex() {
        return this.isTabbingBackOut() || this.getFocusableItemsCount() === 0 ? -1 : 0;
    }
    /** @ignore */
    handleBlur() {
        this.isTabbingBackOut.set(false);
    }
    /** @ignore */
    handleMouseUp() {
        // reset `isClickFocus` after 1 tick because handleFocus might not triggered due to focused element
        this.ngZone.runOutsideAngular(() => {
            // eslint-disable-next-line promise/catch-or-return,promise/always-return
            Promise.resolve().then(() => {
                this.ngZone.run(() => {
                    this.isClickFocus.set(false);
                });
            });
        });
    }
    /** @ignore */
    handleFocus(event) {
        // We normally wouldn't need this check, because we already check
        // that the focus is on the current target and not bubbling to it.
        // We do this because Safari doesn't focus buttons when clicked, and
        // instead, the wrapper will get focused and not through a bubbling event.
        const isKeyboardFocus = !this.isClickFocus();
        if (event.currentTarget === this.elementRef.nativeElement &&
            event.target === event.currentTarget &&
            isKeyboardFocus &&
            !this.isTabbingBackOut()) {
            const entryFocusEvent = new CustomEvent(ENTRY_FOCUS, EVENT_OPTIONS);
            this.elementRef.nativeElement.dispatchEvent(entryFocusEvent);
            this.entryFocus.emit(entryFocusEvent);
            if (!entryFocusEvent.defaultPrevented) {
                const items = this.focusableItems().filter((item) => item.dataset['disabled'] !== '');
                const activeItem = items.find((item) => item.getAttribute('data-active') === 'true');
                const currentItem = items.find((item) => item.id === this.currentTabStopId());
                const candidateItems = [activeItem, currentItem, ...items].filter(Boolean);
                focusFirst(candidateItems, this.preventScrollOnEntryFocus);
            }
        }
        this.isClickFocus.set(false);
    }
    /** @ignore */
    handleMouseDown() {
        this.isClickFocus.set(true);
    }
    /** @ignore */
    onItemFocus(tabStopId) {
        this.currentTabStopId.set(tabStopId);
        this.currentTabStopIdChange.emit(tabStopId);
    }
    /** @ignore */
    onItemShiftTab() {
        this.isTabbingBackOut.set(true);
    }
    /** @ignore */
    onFocusableItemAdd() {
        this.focusableItemsCount.update((count) => count + 1);
    }
    /** @ignore */
    onFocusableItemRemove() {
        this.focusableItemsCount.update((count) => Math.max(0, count - 1));
    }
    /** @ignore */
    registerItem(item) {
        const currentItems = this.focusableItems();
        this.focusableItems.set([...currentItems, item]);
    }
    /** @ignore */
    unregisterItem(item) {
        const currentItems = this.focusableItems();
        this.focusableItems.set(currentItems.filter((el) => el !== item));
    }
    /** @ignore */
    getFocusableItemsCount() {
        return this.focusableItemsCount();
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxRovingFocusGroupDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "16.1.0", version: "19.2.4", type: RdxRovingFocusGroupDirective, isStandalone: true, selector: "[rdxRovingFocusGroup]", inputs: { orientation: "orientation", dir: "dir", loop: ["loop", "loop", booleanAttribute], preventScrollOnEntryFocus: ["preventScrollOnEntryFocus", "preventScrollOnEntryFocus", booleanAttribute] }, outputs: { entryFocus: "entryFocus", currentTabStopIdChange: "currentTabStopIdChange" }, host: { listeners: { "focus": "handleFocus($event)", "blur": "handleBlur()", "mouseup": "handleMouseUp()", "mousedown": "handleMouseDown()" }, properties: { "attr.data-orientation": "dataOrientation", "attr.tabindex": "tabIndex", "attr.dir": "dir" }, styleAttribute: "outline: none;" }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxRovingFocusGroupDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxRovingFocusGroup]',
                    standalone: true,
                    host: {
                        '[attr.data-orientation]': 'dataOrientation',
                        '[attr.tabindex]': 'tabIndex',
                        '[attr.dir]': 'dir',
                        '(focus)': 'handleFocus($event)',
                        '(blur)': 'handleBlur()',
                        '(mouseup)': 'handleMouseUp()',
                        '(mousedown)': 'handleMouseDown()',
                        style: 'outline: none;'
                    }
                }]
        }], propDecorators: { orientation: [{
                type: Input
            }], dir: [{
                type: Input
            }], loop: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], preventScrollOnEntryFocus: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], entryFocus: [{
                type: Output
            }], currentTabStopIdChange: [{
                type: Output
            }] } });

class RdxRovingFocusItemDirective {
    constructor() {
        this.elementRef = inject(ElementRef);
        this.ngZone = inject(NgZone);
        this.parent = inject(RdxRovingFocusGroupDirective);
        this.focusable = true;
        this.active = true;
        this.allowShiftKey = false;
        this.id = computed(() => this.tabStopId || generateId());
        /** @ignore */
        this.isCurrentTabStop = computed(() => this.parent.currentTabStopId() === this.id());
    }
    /**
     * Lifecycle hook triggered on initialization.
     * Registers the element with the parent roving focus group if it is focusable.
     * @ignore
     */
    ngOnInit() {
        if (this.focusable) {
            this.parent.registerItem(this.elementRef.nativeElement);
            this.parent.onFocusableItemAdd();
        }
    }
    /**
     * Lifecycle hook triggered on destruction.
     * Unregisters the element from the parent roving focus group if it is focusable.
     * @ignore
     */
    ngOnDestroy() {
        if (this.focusable) {
            this.parent.unregisterItem(this.elementRef.nativeElement);
            this.parent.onFocusableItemRemove();
        }
    }
    /**
     * Determines the `tabIndex` of the element.
     * Returns `0` if the element is the current tab stop; otherwise, returns `-1`.
     * @ignore
     */
    get tabIndex() {
        return this.isCurrentTabStop() ? 0 : -1;
    }
    /** @ignore */
    handleMouseDown(event) {
        if (!this.focusable) {
            // We prevent focusing non-focusable items on `mousedown`.
            // Even though the item has tabIndex={-1}, that only means take it out of the tab order.
            event.preventDefault();
        }
        else {
            // Safari doesn't focus a button when clicked so we run our logic on mousedown also
            this.parent.onItemFocus(this.id());
        }
    }
    /** @ignore */
    onFocus() {
        this.parent.onItemFocus(this.id());
    }
    /**
     * Handles the `keydown` event for keyboard navigation within the roving focus group.
     * Supports navigation based on orientation and direction, and focuses appropriate elements.
     *
     * @param event The `KeyboardEvent` object.
     * @ignore
     */
    handleKeydown(event) {
        if (event.key === 'Tab' && event.shiftKey) {
            this.parent.onItemShiftTab();
            return;
        }
        if (event.target !== this.elementRef.nativeElement)
            return;
        const focusIntent = getFocusIntent(event, this.parent.orientation, this.parent.dir);
        if (focusIntent !== undefined) {
            if (event.metaKey || event.ctrlKey || event.altKey || (this.allowShiftKey ? false : event.shiftKey)) {
                return;
            }
            event.preventDefault();
            let candidateNodes = this.parent.focusableItems().filter((item) => item.dataset['disabled'] !== '');
            if (focusIntent === 'last') {
                candidateNodes.reverse();
            }
            else if (focusIntent === 'prev' || focusIntent === 'next') {
                if (focusIntent === 'prev')
                    candidateNodes.reverse();
                const currentIndex = candidateNodes.indexOf(this.elementRef.nativeElement);
                candidateNodes = this.parent.loop
                    ? wrapArray(candidateNodes, currentIndex + 1)
                    : candidateNodes.slice(currentIndex + 1);
            }
            this.ngZone.runOutsideAngular(() => {
                // eslint-disable-next-line promise/always-return,promise/catch-or-return
                Promise.resolve().then(() => {
                    focusFirst(candidateNodes, false, this.elementRef.nativeElement);
                });
            });
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxRovingFocusItemDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "16.1.0", version: "19.2.4", type: RdxRovingFocusItemDirective, isStandalone: true, selector: "[rdxRovingFocusItem]", inputs: { focusable: ["focusable", "focusable", booleanAttribute], active: ["active", "active", booleanAttribute], tabStopId: "tabStopId", allowShiftKey: ["allowShiftKey", "allowShiftKey", booleanAttribute] }, host: { listeners: { "mousedown": "handleMouseDown($event)", "keydown": "handleKeydown($event)", "focus": "onFocus()" }, properties: { "attr.tabindex": "tabIndex", "attr.data-orientation": "parent.orientation", "attr.data-active": "active", "attr.data-disabled": "!focusable ? \"\" : undefined" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxRovingFocusItemDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxRovingFocusItem]',
                    standalone: true,
                    host: {
                        '[attr.tabindex]': 'tabIndex',
                        '[attr.data-orientation]': 'parent.orientation',
                        '[attr.data-active]': 'active',
                        '[attr.data-disabled]': '!focusable ? "" : undefined',
                        '(mousedown)': 'handleMouseDown($event)',
                        '(keydown)': 'handleKeydown($event)',
                        '(focus)': 'onFocus()'
                    }
                }]
        }], propDecorators: { focusable: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], active: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }], tabStopId: [{
                type: Input
            }], allowShiftKey: [{
                type: Input,
                args: [{ transform: booleanAttribute }]
            }] } });

/**
 * Generated bundle index. Do not edit.
 */

export { RdxRovingFocusGroupDirective, RdxRovingFocusItemDirective };
//# sourceMappingURL=radix-ng-primitives-roving-focus.mjs.map
