import * as i0 from '@angular/core';
import { InjectionToken, inject, input, computed, signal, effect, Directive, ElementRef, model, booleanAttribute, linkedSignal, NgModule } from '@angular/core';
import { useDateField, normalizeDateStep, getDefaultTime, isNullish, isBefore, createContent, watch, createFormatter, normalizeHourCycle, initializeSegmentValues, syncSegmentValues, getSegmentElements, isSegmentNavigationKey, ARROW_LEFT, ARROW_RIGHT, provideToken } from '@radix-ng/primitives/core';
import { today, toCalendarDateTime, getLocalTimeZone, isEqualDay } from '@internationalized/date';

const TIME_FIELDS_ROOT_CONTEXT = new InjectionToken('TIME_FIELDS_ROOT_CONTEXT');
function injectTimeFieldsRootContext() {
    return inject(TIME_FIELDS_ROOT_CONTEXT);
}

class RdxTimeFieldInputDirective {
    constructor(el) {
        this.el = el;
        this.rootContext = injectTimeFieldsRootContext();
        /**
         * The part of the date to render
         * `'hour' | 'minute' | 'second' | 'dayPeriod' | 'literal' | 'timeZoneName'`
         */
        this.part = input();
        /**
         * @ignore
         */
        this.disabled = computed(() => this.rootContext.disabled());
        /**
         * @ignore
         */
        this.readonly = computed(() => this.rootContext.readonly());
        /**
         * @ignore
         */
        this.isInvalid = computed(() => this.rootContext.isInvalid());
        /**
         * @ignore
         */
        this.hasLeftFocus = signal(true);
        /**
         * @ignore
         */
        this.lastKeyZero = signal(false);
        this.fieldData = computed(() => {
            return useDateField({
                hasLeftFocus: this.hasLeftFocus,
                lastKeyZero: this.lastKeyZero,
                placeholder: this.rootContext.convertedPlaceholder,
                hourCycle: this.rootContext.hourCycle(),
                segmentValues: this.rootContext.segmentValues,
                formatter: this.rootContext.formatter,
                part: this.part(),
                disabled: this.rootContext.disabled,
                readonly: this.rootContext.readonly,
                modelValue: this.rootContext.convertedModelValue,
                focusNext: this.rootContext.focusNext,
                step: this.rootContext.step$
            });
        });
        this.attributes = computed(() => this.fieldData().attributes());
        effect(() => {
            const { handleSegmentClick, handleSegmentKeydown } = this.fieldData();
            this.handleSegmentKeydown = handleSegmentKeydown;
            this.handleSegmentClick = handleSegmentClick;
        });
        effect(() => {
            const attrs = this.attributes();
            Object.entries(attrs).forEach(([attr, value]) => {
                this.el.nativeElement.setAttribute(attr, String(value));
            });
        });
    }
    /**
     * @ignore
     */
    onFocus(e) {
        this.rootContext.setFocusedElement(e.target);
    }
    /**
     * @ignore
     */
    onFocusOut() {
        this.hasLeftFocus.set(true);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldInputDirective, deps: [{ token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxTimeFieldInputDirective, isStandalone: true, selector: "[rdxTimeFieldInput]", inputs: { part: { classPropertyName: "part", publicName: "part", isSignal: true, isRequired: false, transformFunction: null } }, host: { listeners: { "mousedown": "part() !== \"literal\" && handleSegmentClick($event)", "keydown": "part() !== \"literal\" && handleSegmentKeydown($event)", "focus": "part() !== \"literal\" && onFocus($event)", "focusout": "part() !== \"literal\" && onFocusOut()" }, properties: { "attr.contenteditable": "disabled() || readonly() ? false : part() !== \"literal\"", "attr.data-rdx-date-field-segment": "part()", "attr.aria-disabled": "disabled() ? \"\" : undefined", "attr.data-disabled": "disabled() ? \"\" : undefined", "attr.data-invalid": "isInvalid() ? \"\" : undefined", "attr.aria-invalid": "isInvalid() ? true : undefined" } }, ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldInputDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxTimeFieldInput]',
                    host: {
                        '[attr.contenteditable]': 'disabled() || readonly() ? false : part() !== "literal"',
                        '[attr.data-rdx-date-field-segment]': 'part()',
                        '[attr.aria-disabled]': 'disabled() ? "" : undefined',
                        '[attr.data-disabled]': 'disabled() ? "" : undefined',
                        '[attr.data-invalid]': 'isInvalid() ? "" : undefined',
                        '[attr.aria-invalid]': 'isInvalid() ? true : undefined',
                        '(mousedown)': 'part() !== "literal" && handleSegmentClick($event)',
                        '(keydown)': 'part() !== "literal" && handleSegmentKeydown($event)',
                        '(focus)': 'part() !== "literal" && onFocus($event)',
                        '(focusout)': 'part() !== "literal" && onFocusOut()'
                    }
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }] });

function convertValue(value, date = today(getLocalTimeZone())) {
    if (value && 'day' in value) {
        return value;
    }
    return toCalendarDateTime(date, value);
}
class RdxTimeFieldRootDirective {
    constructor() {
        this.elementRef = inject((ElementRef));
        /**
         * The controlled checked state of the calendar.
         */
        this.value = model();
        /**
         * The hour cycle to use for formatting times. Defaults to the locale preference
         */
        this.hourCycle = input();
        /**
         * The granularity to use for formatting the field. Defaults to 'day' if a CalendarDate is provided, otherwise defaults to 'minute'.
         * The field will render segments for each part of the date up to and including the specified granularity.
         */
        this.granularity = input();
        /**
         * The locale to use for formatting dates.
         */
        this.locale = input('en');
        this.dir = input('ltr');
        /**
         * The minimum valid date that can be entered.
         */
        this.minValue = input();
        /**
         * The maximum valid date that can be entered.
         */
        this.maxValue = input();
        /**
         * Whether or not to hide the time zone segment of the field.
         */
        this.hideTimeZone = input(false, { transform: booleanAttribute });
        this.disabled = input(false, { transform: booleanAttribute });
        /**
         * Whether or not the field is readonly.
         */
        this.readonly = input(false, { transform: booleanAttribute });
        /**
         * The stepping interval for the time fields. Defaults to 1
         */
        this.step = input();
        this.step$ = computed(() => normalizeDateStep(this.step()));
        /**
         * @ignore
         */
        this.defaultDate = computed(() => getDefaultTime({
            defaultPlaceholder: undefined,
            defaultValue: this.value()
        }));
        /**
         * The placeholder date, which is used to determine what month to display when no date is selected. This updates as the user navigates the calendar and can be used to programmatically control the calendar view
         */
        this.placeholder = model(this.defaultDate().copy());
        // Internal state
        /**
         * @ignore
         */
        this.segmentElements = signal(new Set());
        /**
         * @ignore
         */
        this.currentFocusedElement = signal(null);
        /**
         * @ignore
         */
        this.segmentValues = signal({
            hour: null,
            minute: null,
            second: null,
            dayPeriod: null
        });
        /**
         * @ignore
         */
        this.inferredGranularity = computed(() => {
            if (this.granularity())
                return this.granularity();
            return 'minute';
        });
        this.convertedMinValue = computed(() => (this.minValue() ? convertValue(this.minValue()) : undefined));
        this.convertedMaxValue = computed(() => (this.maxValue() ? convertValue(this.maxValue()) : undefined));
        this.convertedModelValue = linkedSignal({
            source: () => {
                if (isNullish(this.value()))
                    return this.value();
                return convertValue(this.value());
            },
            computation: (value) => {
                return value;
            }
        });
        this.convertedPlaceholder = linkedSignal({
            source: () => {
                return convertValue(this.placeholder());
            },
            computation: (value) => {
                return value;
            }
        });
        /**
         * @ignore
         */
        this.isInvalid = computed(() => {
            if (!this.value())
                return false;
            if (this.convertedMinValue() &&
                isBefore(this.convertedModelValue(), this.convertedMinValue()))
                return true;
            if (this.convertedMaxValue() &&
                isBefore(this.convertedMaxValue(), this.convertedModelValue()))
                return true;
            return false;
        });
        /**
         * @ignore
         */
        this.allSegmentContent = computed(() => createContent({
            granularity: this.inferredGranularity(),
            dateRef: this.convertedPlaceholder(),
            formatter: this.formatter,
            hideTimeZone: this.hideTimeZone(),
            hourCycle: this.hourCycle(),
            segmentValues: this.segmentValues(),
            locale: this.locale,
            isTimeValue: true
        }));
        /**
         * An array of segments that should be readonly, which prevent user input on them.
         */
        this.segmentContents = computed(() => this.allSegmentContent().arr);
        /**
         * @ignore
         */
        this.currentSegmentIndex = computed(() => Array.from(this.segmentElements()).findIndex((el) => el.getAttribute('data-rdx-date-field-segment') ===
            this.currentFocusedElement()?.getAttribute('data-rdx-date-field-segment')));
        /**
         * @ignore
         */
        this.prevFocusableSegment = computed(() => {
            const sign = this.dir() === 'rtl' ? -1 : 1;
            const prevCondition = sign > 0 ? this.currentSegmentIndex() < 0 : this.currentSegmentIndex() > this.segmentElements().size - 1;
            if (prevCondition)
                return null;
            const segmentToFocus = Array.from(this.segmentElements())[this.currentSegmentIndex() - sign];
            return segmentToFocus;
        });
        /**
         * @ignore
         */
        this.nextFocusableSegment = computed(() => {
            const sign = this.dir() === 'rtl' ? -1 : 1;
            const nextCondition = sign < 0 ? this.currentSegmentIndex() < 0 : this.currentSegmentIndex() > this.segmentElements().size - 1;
            if (nextCondition)
                return null;
            const segmentToFocus = Array.from(this.segmentElements())[this.currentSegmentIndex() + sign];
            return segmentToFocus;
        });
        /**
         * @ignore
         */
        this.focusNext = () => {
            this.nextFocusableSegment()?.focus();
        };
        watch([this.convertedModelValue], ([value]) => {
            if (!isNullish(value) &&
                (!isEqualDay(this.convertedPlaceholder(), value) ||
                    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                    // @ts-expect-error
                    this.convertedPlaceholder().compare(value) !== 0))
                this.placeholder.set(value.copy());
        });
    }
    ngOnInit() {
        const defDate = getDefaultTime({
            defaultPlaceholder: undefined,
            defaultValue: this.value()
        });
        this.placeholder.set(defDate.copy());
        this.formatter = createFormatter(this.locale(), {
            hourCycle: normalizeHourCycle(this.hourCycle())
        });
        const initialSegments = initializeSegmentValues(this.inferredGranularity());
        this.segmentValues.set(this.value()
            ? { ...syncSegmentValues({ value: this.value(), formatter: this.formatter }) }
            : { ...initialSegments });
    }
    ngAfterViewInit() {
        getSegmentElements(this.elementRef.nativeElement).forEach((item) => this.segmentElements().add(item));
    }
    /**
     * @ignore
     */
    onKeydown(event) {
        if (!isSegmentNavigationKey(event.key))
            return;
        const code = event.code;
        if (code === ARROW_LEFT) {
            this.prevFocusableSegment()?.focus();
        }
        if (code === ARROW_RIGHT) {
            this.nextFocusableSegment()?.focus();
        }
    }
    /**
     * @ignore
     */
    setFocusedElement(el) {
        this.currentFocusedElement.set(el);
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldRootDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive }); }
    static { this.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "17.1.0", version: "19.2.4", type: RdxTimeFieldRootDirective, isStandalone: true, selector: "[rdxTimeFieldRoot]", inputs: { value: { classPropertyName: "value", publicName: "value", isSignal: true, isRequired: false, transformFunction: null }, hourCycle: { classPropertyName: "hourCycle", publicName: "hourCycle", isSignal: true, isRequired: false, transformFunction: null }, granularity: { classPropertyName: "granularity", publicName: "granularity", isSignal: true, isRequired: false, transformFunction: null }, locale: { classPropertyName: "locale", publicName: "locale", isSignal: true, isRequired: false, transformFunction: null }, dir: { classPropertyName: "dir", publicName: "dir", isSignal: true, isRequired: false, transformFunction: null }, minValue: { classPropertyName: "minValue", publicName: "minValue", isSignal: true, isRequired: false, transformFunction: null }, maxValue: { classPropertyName: "maxValue", publicName: "maxValue", isSignal: true, isRequired: false, transformFunction: null }, hideTimeZone: { classPropertyName: "hideTimeZone", publicName: "hideTimeZone", isSignal: true, isRequired: false, transformFunction: null }, disabled: { classPropertyName: "disabled", publicName: "disabled", isSignal: true, isRequired: false, transformFunction: null }, readonly: { classPropertyName: "readonly", publicName: "readonly", isSignal: true, isRequired: false, transformFunction: null }, step: { classPropertyName: "step", publicName: "step", isSignal: true, isRequired: false, transformFunction: null }, placeholder: { classPropertyName: "placeholder", publicName: "placeholder", isSignal: true, isRequired: false, transformFunction: null } }, outputs: { value: "valueChange", placeholder: "placeholderChange" }, host: { attributes: { "role": "group" }, listeners: { "keydown": "onKeydown($event)" }, properties: { "attr.aria-disabled": "disabled() ? \"\" : undefined", "attr.data-disabled": "disabled() ? \"\" : undefined", "attr.data-readonly": "readonly() ? \"\" : undefined", "attr.data-invalid": "isInvalid() ? \"\" : undefined", "attr.dir": "dir()" } }, providers: [provideToken(TIME_FIELDS_ROOT_CONTEXT, RdxTimeFieldRootDirective)], exportAs: ["rdxTimeFieldRoot"], ngImport: i0 }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldRootDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[rdxTimeFieldRoot]',
                    exportAs: 'rdxTimeFieldRoot',
                    providers: [provideToken(TIME_FIELDS_ROOT_CONTEXT, RdxTimeFieldRootDirective)],
                    host: {
                        role: 'group',
                        '[attr.aria-disabled]': 'disabled() ? "" : undefined',
                        '[attr.data-disabled]': 'disabled() ? "" : undefined',
                        '[attr.data-readonly]': 'readonly() ? "" : undefined',
                        '[attr.data-invalid]': 'isInvalid() ? "" : undefined',
                        '[attr.dir]': 'dir()',
                        '(keydown)': 'onKeydown($event)'
                    }
                }]
        }], ctorParameters: () => [] });

const _imports = [RdxTimeFieldRootDirective, RdxTimeFieldInputDirective];
class RdxTimeFieldModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldModule, imports: [RdxTimeFieldRootDirective, RdxTimeFieldInputDirective], exports: [RdxTimeFieldRootDirective, RdxTimeFieldInputDirective] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.2.4", ngImport: i0, type: RdxTimeFieldModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [..._imports],
                    exports: [..._imports]
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { RdxTimeFieldInputDirective, RdxTimeFieldModule, RdxTimeFieldRootDirective, TIME_FIELDS_ROOT_CONTEXT, injectTimeFieldsRootContext };
//# sourceMappingURL=radix-ng-primitives-time-field.mjs.map
